# System modules

# allow the use of the `list[str]` type hint in Python 3.8
from __future__ import annotations

from collections.abc import Callable
from functools import wraps
from packaging import version
import contextlib
import ctypes
import locale
import os
import platform
import re
import sys
import tempfile
import subprocess
import json

# Third-party modules
import unittest

# LLDB modules
import lldb
from . import configuration
from . import test_categories
from . import lldbtest_config
from lldbsuite.support import funcutils
from lldbsuite.support import temp_file
from lldbsuite.test import lldbplatform
from lldbsuite.test import lldbplatformutil
from lldbsuite.test.cpu_feature import CPUFeature


class DecorateMode:
    Skip, Xfail = range(2)


# You can use no_match to reverse the test of the conditional that is used to match keyword
# arguments in the skip / xfail decorators.  If oslist=["windows", "linux"] skips windows
# and linux, oslist=no_match(["windows", "linux"]) skips *unless* windows
# or linux.
class no_match:
    def __init__(self, item):
        self.item = item


def _check_expected_version(comparison, expected, actual):
    def fn_leq(x, y):
        return x <= y

    def fn_less(x, y):
        return x < y

    def fn_geq(x, y):
        return x >= y

    def fn_greater(x, y):
        return x > y

    def fn_eq(x, y):
        return x == y

    def fn_neq(x, y):
        return x != y

    op_lookup = {
        "==": fn_eq,
        "=": fn_eq,
        "!=": fn_neq,
        "<>": fn_neq,
        ">": fn_greater,
        "<": fn_less,
        ">=": fn_geq,
        "<=": fn_leq,
    }

    return op_lookup[comparison](version.parse(actual), version.parse(expected))


def _match_decorator_property(expected, actual):
    if expected is None:
        return True

    if actual is None:
        return False

    if isinstance(expected, no_match):
        return not _match_decorator_property(expected.item, actual)

    # Python 3.6 doesn't declare a `re.Pattern` type, get the dynamic type.
    pattern_type = type(re.compile(""))
    if isinstance(expected, (pattern_type, str)):
        return re.search(expected, actual) is not None

    if hasattr(expected, "__iter__"):
        return any(
            [x is not None and _match_decorator_property(x, actual) for x in expected]
        )

    return expected == actual


def _compiler_supports(compiler, flag, source="int main() {}", output_file=None):
    """Test whether the compiler supports the given flag."""
    if output_file:
        context = contextlib.nullcontext(output_file)
    else:
        context = temp_file.OnDiskTempFile()
    with context as ctx:
        if platform.system() == "Darwin":
            compiler = "xcrun " + compiler
        try:
            cmd = "echo '%s' | %s %s -x c -o %s -" % (
                source,
                compiler,
                flag,
                ctx.path,
            )
            subprocess.check_call(cmd, shell=True)
        except subprocess.CalledProcessError:
            return False
        return True


def expectedFailureIf(condition, bugnumber=None):
    def expectedFailure_impl(func):
        if isinstance(func, type) and issubclass(func, unittest.TestCase):
            raise Exception("Decorator can only be used to decorate a test method")

        if condition:
            return unittest.expectedFailure(func)
        return func

    if callable(bugnumber):
        return expectedFailure_impl(bugnumber)
    else:
        return expectedFailure_impl


def skipTestIfFn(expected_fn, bugnumber=None):
    def skipTestIfFn_impl(func):
        if isinstance(func, type) and issubclass(func, unittest.TestCase):
            reason = expected_fn()
            # The return value is the reason (or None if we don't skip), so
            # reason is used for both args.
            return unittest.skipIf(condition=reason, reason=reason)(func)

        @wraps(func)
        def wrapper(*args, **kwargs):
            self = args[0]
            if funcutils.requires_self(expected_fn):
                reason = expected_fn(self)
            else:
                reason = expected_fn()

            if reason is not None:
                self.skipTest(reason)
            else:
                return func(*args, **kwargs)

        return wrapper

    # Some decorators can be called both with no arguments (e.g. @expectedFailureWindows)
    # or with arguments (e.g. @expectedFailureWindows(compilers=['gcc'])).  When called
    # the first way, the first argument will be the actual function because decorators are
    # weird like that.  So this is basically a check that says "how was the
    # decorator used"
    if callable(bugnumber):
        return skipTestIfFn_impl(bugnumber)
    else:
        return skipTestIfFn_impl


def _xfailForVariant(variant_name, expected_fn, bugnumber=None):
    """Mark a test method as expected-failure for a specific variant dimension.

    Adds *expected_fn* to the decorated function's `__variant_xfail__`
    dictionary under the key *variant_name*.  When the `LLDBTestCaseFactory`
    metaclass expands the test method, it looks up *variant_name* in this
    dictionary and calls the corresponding function with the concrete variant
    value.  If the function returns a reason string, the expanded test method
    is marked as expected failure via `unittest.expectedFailure`.

    Args:
        variant_name: The variant dimension name (e.g. `"debug_info"`).
        expected_fn: Callable `(**{variant_name: value}) -> reason | None`.
        bugnumber: Optional bug reference or the decorated function itself
            (when the decorator is used without parentheses).
    """
    def expectedFailure_impl(func):
        if isinstance(func, type) and issubclass(func, unittest.TestCase):
            raise Exception("Decorator can only be used to decorate a test method")
        xfail_dict = getattr(func, "__variant_xfail__", {})
        xfail_dict[variant_name] = expected_fn
        func.__variant_xfail__ = xfail_dict
        return func

    if callable(bugnumber):
        return expectedFailure_impl(bugnumber)
    else:
        return expectedFailure_impl


def _skipForVariant(variant_name, expected_fn, bugnumber=None):
    """Mark a test method as skipped for a specific variant dimension.

    Adds *expected_fn* to the decorated function's `__variant_skip__`
    dictionary under the key *variant_name*.  When the `LLDBTestCaseFactory`
    metaclass expands the test method, it looks up *variant_name* in this
    dictionary and calls the corresponding function with the concrete variant
    value.  If the function returns a reason string, the expanded test method
    is skipped via `unittest.skip(reason)`.

    Args:
        variant_name: The variant dimension name (e.g. `"debug_info"`).
        expected_fn: Callable `(**{variant_name: value}) -> reason | None`.
        bugnumber: Optional bug reference or the decorated function itself
            (when the decorator is used without parentheses).
    """
    def skipImpl(func):
        if isinstance(func, type) and issubclass(func, unittest.TestCase):
            raise Exception("Decorator can only be used to decorate a test method")
        skip_dict = getattr(func, "__variant_skip__", {})
        skip_dict[variant_name] = expected_fn
        func.__variant_skip__ = skip_dict
        return func

    if callable(bugnumber):
        return skipImpl(bugnumber)
    else:
        return skipImpl


def _decorateTest(
    mode,
    bugnumber=None,
    oslist=None,
    hostoslist=None,
    compiler=None,
    compiler_version=None,
    archs=None,
    triple=None,
    debug_info=None,
    swig_version=None,
    py_version=None,
    macos_version=None,
    remote=None,
    dwarf_version=None,
    setting=None,
    asan=None,
):
    def fn(**actual_variants):
        skip_for_os = _match_decorator_property(
            lldbplatform.translate(oslist), lldbplatformutil.getPlatform()
        )
        skip_for_hostos = _match_decorator_property(
            lldbplatform.translate(hostoslist), lldbplatformutil.getHostPlatform()
        )
        skip_for_compiler = _match_decorator_property(
            compiler, lldbplatformutil.getCompiler()
        ) and lldbplatformutil.expectedCompilerVersion(compiler_version)
        skip_for_arch = _match_decorator_property(
            archs, lldbplatformutil.getArchitecture()
        )
        skip_for_debug_info = _match_decorator_property(
            debug_info, actual_variants.get("debug_info")
        )
        skip_for_triple = _match_decorator_property(
            triple, lldb.selected_platform.GetTriple()
        )
        skip_for_remote = _match_decorator_property(
            remote, lldb.remote_platform is not None
        )

        skip_for_swig_version = (
            (swig_version is None)
            or (not hasattr(lldb, "swig_version"))
            or (
                _check_expected_version(
                    swig_version[0], swig_version[1], lldb.swig_version
                )
            )
        )
        skip_for_py_version = (py_version is None) or _check_expected_version(
            py_version[0],
            py_version[1],
            "{}.{}".format(sys.version_info.major, sys.version_info.minor),
        )
        skip_for_macos_version = (macos_version is None) or (
            (platform.mac_ver()[0] != "")
            and (
                _check_expected_version(
                    macos_version[0], macos_version[1], platform.mac_ver()[0]
                )
            )
        )
        skip_for_dwarf_version = (dwarf_version is None) or (
            _check_expected_version(
                dwarf_version[0], dwarf_version[1], lldbplatformutil.getDwarfVersion()
            )
        )
        skip_for_setting = (setting is None) or (setting in configuration.settings)
        skip_for_asan = (asan is None) or is_running_under_asan()

        # For the test to be skipped, all specified (e.g. not None) parameters must be True.
        # An unspecified parameter means "any", so those are marked skip by default.  And we skip
        # the final test if all conditions are True.
        conditions = [
            (oslist, skip_for_os, "target o/s"),
            (hostoslist, skip_for_hostos, "host o/s"),
            (compiler, skip_for_compiler, "compiler or version"),
            (archs, skip_for_arch, "architecture"),
            (debug_info, skip_for_debug_info, "debug info format"),
            (triple, skip_for_triple, "target triple"),
            (swig_version, skip_for_swig_version, "swig version"),
            (py_version, skip_for_py_version, "python version"),
            (macos_version, skip_for_macos_version, "macOS version"),
            (remote, skip_for_remote, "platform locality (remote/local)"),
            (dwarf_version, skip_for_dwarf_version, "dwarf version"),
            (setting, skip_for_setting, "setting"),
            (asan, skip_for_asan, "running under asan"),
        ]
        reasons = []
        final_skip_result = True
        for this_condition in conditions:
            final_skip_result = final_skip_result and this_condition[1]
            if this_condition[0] is not None and this_condition[1]:
                reasons.append(this_condition[2])
        reason_str = None
        if final_skip_result:
            mode_str = {DecorateMode.Skip: "skipping", DecorateMode.Xfail: "xfailing"}[
                mode
            ]
            if len(reasons) > 0:
                reason_str = ",".join(reasons)
                reason_str = "{} due to the following parameter(s): {}".format(
                    mode_str, reason_str
                )
            else:
                reason_str = "{} unconditionally".format(mode_str)
            if bugnumber is not None and not callable(bugnumber):
                reason_str = reason_str + " [" + str(bugnumber) + "]"
        return reason_str

    if mode == DecorateMode.Skip:
        if debug_info:
            return _skipForVariant("debug_info", fn, bugnumber)
        return skipTestIfFn(fn, bugnumber)
    elif mode == DecorateMode.Xfail:
        if debug_info:
            return _xfailForVariant("debug_info", fn, bugnumber)
        return expectedFailureIf(fn(), bugnumber)
    else:
        return None


# provide a function to xfail on defined oslist, compiler version, and archs
# if none is specified for any argument, that argument won't be checked and thus means for all
# for example,
# @expectedFailureAll, xfail for all platform/compiler/arch,
# @expectedFailureAll(compiler='gcc'), xfail for gcc on all platform/architecture
# @expectedFailureAll(bugnumber, ["linux"], "gcc", ['>=', '4.9'], ['i386']), xfail for gcc>=4.9 on linux with i386


def expectedFailureAll(
    bugnumber=None,
    oslist=None,
    hostoslist=None,
    compiler=None,
    compiler_version=None,
    archs=None,
    triple=None,
    debug_info=None,
    swig_version=None,
    py_version=None,
    macos_version=None,
    remote=None,
    dwarf_version=None,
    setting=None,
    asan=None,
):
    return _decorateTest(
        DecorateMode.Xfail,
        bugnumber=bugnumber,
        oslist=oslist,
        hostoslist=hostoslist,
        compiler=compiler,
        compiler_version=compiler_version,
        archs=archs,
        triple=triple,
        debug_info=debug_info,
        swig_version=swig_version,
        py_version=py_version,
        macos_version=macos_version,
        remote=remote,
        dwarf_version=dwarf_version,
        setting=setting,
        asan=asan,
    )


# provide a function to skip on defined oslist, compiler version, and archs
# if none is specified for any argument, that argument won't be checked and thus means for all
# for example,
# @skipIf, skip for all platform/compiler/arch,
# @skipIf(compiler='gcc'), skip for gcc on all platform/architecture
# @skipIf(bugnumber, ["linux"], "gcc", ['>=', '4.9'], ['i386']), skip for gcc>=4.9 on linux with i386 (all conditions must be true)
def skipIf(
    bugnumber=None,
    oslist=None,
    hostoslist=None,
    compiler=None,
    compiler_version=None,
    archs=None,
    triple=None,
    debug_info=None,
    swig_version=None,
    py_version=None,
    macos_version=None,
    remote=None,
    dwarf_version=None,
    setting=None,
    asan=None,
):
    return _decorateTest(
        DecorateMode.Skip,
        bugnumber=bugnumber,
        oslist=oslist,
        hostoslist=hostoslist,
        compiler=compiler,
        compiler_version=compiler_version,
        archs=archs,
        triple=triple,
        debug_info=debug_info,
        swig_version=swig_version,
        py_version=py_version,
        macos_version=macos_version,
        remote=remote,
        dwarf_version=dwarf_version,
        setting=setting,
        asan=asan,
    )


def skip(bugnumber=None):
    return _decorateTest(DecorateMode.Skip, bugnumber=bugnumber)


def _skip_fn_for_android(reason, api_levels, archs):
    def impl():
        result = lldbplatformutil.match_android_device(
            lldbplatformutil.getArchitecture(),
            valid_archs=archs,
            valid_api_levels=api_levels,
        )
        return reason if result else None

    return impl


def add_test_categories(cat):
    """Add test categories to a TestCase method"""
    cat = test_categories.validate(cat, True)

    def impl(func):
        try:
            if hasattr(func, "categories"):
                cat.extend(func.categories)
            setattr(func, "categories", cat)
        except AttributeError:
            raise Exception("Cannot assign categories to inline tests.")

        return func

    return impl


def unicode_test(func):
    """Decorate the item as a test which requires Unicode to be enabled.

    lldb checks the value of the `LANG` environment variable for the substring "utf-8"
    to determine if the terminal supports Unicode (except on Windows, where we assume
    it's always supported).
    This decorator sets LANG to `utf-8` before running the test and resets it to its
    previous value afterwards.
    """

    if sys.platform == "win32":
        # Unicode support on Windows is flaky in CI.
        return expectedFailureWindows

    def unicode_wrapped(*args, **kwargs):
        import os

        previous_lang = os.environ.get("LANG", None)
        os.environ["LANG"] = "en_US.UTF-8"
        try:
            func(*args, **kwargs)
        except Exception as err:
            raise err
        finally:
            # Reset the value, whether the test failed or not.
            if previous_lang is not None:
                os.environ["LANG"] = previous_lang
            else:
                del os.environ["LANG"]

    return unicode_wrapped


def no_debug_info_test(func):
    """Decorate the item as a test what don't use any debug info. If this annotation is specified
    then the test runner won't generate a separate test for each debug info format."""
    if isinstance(func, type) and issubclass(func, unittest.TestCase):
        raise Exception(
            "@no_debug_info_test can only be used to decorate a test method"
        )

    @wraps(func)
    def wrapper(self, *args, **kwargs):
        return func(self, *args, **kwargs)

    # Mark this function as such to separate them from the regular tests.
    wrapper.__no_debug_info_test__ = True
    return wrapper


def apple_simulator_test(platform):
    """
    Decorate the test as a test requiring a simulator for a specific platform.

    Consider that a simulator is available if you have the corresponding SDK
    and runtime installed.

    The SDK identifiers for simulators are iphonesimulator, appletvsimulator,
    watchsimulator
    """

    def should_skip_simulator_test():
        if lldbplatformutil.getHostPlatform() not in ["darwin", "macosx"]:
            return "simulator tests are run only on darwin hosts."

        # Make sure we recognize the platform.
        mapping = {
            "iphone": "ios",
            "appletv": "tvos",
            "watch": "watchos",
        }
        if platform not in mapping:
            return "unknown simulator platform: {}".format(platform)

        # Make sure we have an SDK.
        try:
            output = subprocess.check_output(
                ["xcodebuild", "-showsdks"], stderr=subprocess.DEVNULL
            ).decode("utf-8")
            if not re.search("%ssimulator" % platform, output):
                return "%s simulator is not supported on this system." % platform
        except subprocess.CalledProcessError:
            return "Simulators are unsupported on this system (xcodebuild failed)"

        # Make sure we a simulator runtime.
        try:
            sim_devices_str = subprocess.check_output(
                ["xcrun", "simctl", "list", "-j", "devices"]
            ).decode("utf-8")

            sim_devices = json.loads(sim_devices_str)["devices"]
            for simulator in sim_devices:
                if isinstance(simulator, dict):
                    runtime = simulator["name"]
                    devices = simulator["devices"]
                else:
                    runtime = simulator
                    devices = sim_devices[simulator]

                if not mapping[platform] in runtime.lower():
                    continue

                for device in devices:
                    if (
                        "availability" in device
                        and device["availability"] == "(available)"
                    ):
                        return None
                    if "isAvailable" in device and device["isAvailable"]:
                        return None

            return "{} simulator is not supported on this system.".format(platform)
        except (subprocess.CalledProcessError, json.decoder.JSONDecodeError):
            return "Simulators are unsupported on this system (simctl failed)"

    return skipTestIfFn(should_skip_simulator_test)


def debugserver_test(func):
    """Decorate the item as a debugserver test."""
    return add_test_categories(["debugserver"])(func)


def llgs_test(func):
    """Decorate the item as a lldb-server test."""
    return add_test_categories(["llgs"])(func)


def expectedFailureOS(
    oslist, bugnumber=None, compilers=None, debug_info=None, archs=None
):
    return expectedFailureAll(
        oslist=oslist,
        bugnumber=bugnumber,
        compiler=compilers,
        archs=archs,
        debug_info=debug_info,
    )


def expectedFailureDarwin(bugnumber=None, compilers=None, debug_info=None, archs=None):
    # For legacy reasons, we support both "darwin" and "macosx" as OS X
    # triples.
    return expectedFailureOS(
        lldbplatform.darwin_all,
        bugnumber,
        compilers,
        debug_info=debug_info,
        archs=archs,
    )


def expectedFailureAndroid(bugnumber=None, api_levels=None, archs=None):
    """Mark a test as xfail for Android.

    Arguments:
        bugnumber - The LLVM pr associated with the problem.
        api_levels - A sequence of numbers specifying the Android API levels
            for which a test is expected to fail. None means all API level.
        arch - A sequence of architecture names specifying the architectures
            for which a test is expected to fail. None means all architectures.
    """
    return expectedFailureIf(
        _skip_fn_for_android("xfailing on android", api_levels, archs)(), bugnumber
    )


def expectedFailureNetBSD(bugnumber=None):
    return expectedFailureOS(["netbsd"], bugnumber)


def expectedFailureWindows(bugnumber=None):
    return expectedFailureOS(["windows"], bugnumber)


# TODO: This decorator does not do anything. Remove it.
def expectedFlakey(expected_fn, bugnumber=None):
    def expectedFailure_impl(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            func(*args, **kwargs)

        return wrapper

    # Some decorators can be called both with no arguments (e.g. @expectedFailureWindows)
    # or with arguments (e.g. @expectedFailureWindows(compilers=['gcc'])).  When called
    # the first way, the first argument will be the actual function because decorators are
    # weird like that.  So this is basically a check that says "which syntax was the original
    # function decorated with?"
    if callable(bugnumber):
        return expectedFailure_impl(bugnumber)
    else:
        return expectedFailure_impl


def expectedFlakeyOS(oslist, bugnumber=None, compilers=None):
    def fn(self):
        return (
            lldbplatformutil.getPlatform() in oslist
            and lldbplatformutil.expectedCompiler(compilers)
        )

    return expectedFlakey(fn, bugnumber)


def expectedFlakeyDarwin(bugnumber=None, compilers=None):
    # For legacy reasons, we support both "darwin" and "macosx" as OS X
    # triples.
    return expectedFlakeyOS(lldbplatformutil.getDarwinOSTriples(), bugnumber, compilers)


def expectedFlakeyFreeBSD(bugnumber=None, compilers=None):
    return expectedFlakeyOS(["freebsd"], bugnumber, compilers)


def expectedFlakeyLinux(bugnumber=None, compilers=None):
    return expectedFlakeyOS(["linux"], bugnumber, compilers)


def expectedFlakeyNetBSD(bugnumber=None, compilers=None):
    return expectedFlakeyOS(["netbsd"], bugnumber, compilers)


def expectedFlakeyAndroid(bugnumber=None, api_levels=None, archs=None):
    return expectedFlakey(
        _skip_fn_for_android("flakey on android", api_levels, archs), bugnumber
    )


def skipIfOutOfTreeDebugserver(func):
    """Decorate the item to skip tests if using an out-of-tree debugserver."""

    def is_out_of_tree_debugserver():
        return (
            "out-of-tree debugserver"
            if lldbtest_config.out_of_tree_debugserver
            else None
        )

    return skipTestIfFn(is_out_of_tree_debugserver)(func)


def skipIfOutOfTreeLibunwind(func):
    """Decorate the item to skip tests if libunwind was not built in-tree."""

    def is_out_of_tree_libunwind():
        if not configuration.llvm_tools_dir:
            return "out-of-tree libunwind"

        # llvm_tools_dir is typically <build>/bin, so lib is a sibling.
        llvm_lib_dir = os.path.join(
            os.path.dirname(configuration.llvm_tools_dir), "lib"
        )

        if not os.path.isdir(llvm_lib_dir):
            return "out-of-tree libunwind"

        # Check for libunwind library (any extension).
        for filename in os.listdir(llvm_lib_dir):
            if filename.startswith("libunwind.") or filename.startswith("unwind."):
                return None

        return "out-of-tree libunwind"

    return skipTestIfFn(is_out_of_tree_libunwind)(func)


def skipIfRemote(func):
    """Decorate the item to skip tests if testing remotely."""
    return unittest.skipIf(lldb.remote_platform, "skip on remote platform")(func)


def skipIfNoSBHeaders(func):
    """Decorate the item to mark tests that should be skipped when LLDB is built with no SB API headers."""

    def are_sb_headers_missing():
        if lldb.remote_platform:
            return "skip because SBHeaders tests make no sense remotely"

        if (
            lldbplatformutil.getHostPlatform() == "darwin"
            and configuration.lldb_framework_path
        ):
            header = os.path.join(
                configuration.lldb_framework_path,
                "Versions",
                "Current",
                "Headers",
                "LLDB.h",
            )
            if os.path.exists(header):
                return None

        header = os.path.join(
            os.environ["LLDB_SRC"], "include", "lldb", "API", "LLDB.h"
        )
        if not os.path.exists(header):
            return "skip because LLDB.h header not found"
        return None

    return skipTestIfFn(are_sb_headers_missing)(func)


def skipIfRosetta(bugnumber):
    """Skip a test when running the testsuite on macOS under the Rosetta translation layer."""

    def is_running_rosetta():
        if lldbplatformutil.getPlatform() in ["darwin", "macosx"]:
            if (platform.uname()[5] == "arm") and (
                lldbplatformutil.getArchitecture() == "x86_64"
            ):
                return "skipped under Rosetta"
        return None

    return skipTestIfFn(is_running_rosetta)


def skipIfiOSSimulator(func):
    """Decorate the item to skip tests that should be skipped on the iOS Simulator."""

    def is_ios_simulator():
        return (
            "skip on the iOS Simulator"
            if configuration.lldb_platform_name == "ios-simulator"
            else None
        )

    return skipTestIfFn(is_ios_simulator)(func)


def skipIfiOS(func):
    return skipIfPlatform(lldbplatform.translate(lldbplatform.ios))(func)


def skipIftvOS(func):
    return skipIfPlatform(lldbplatform.translate(lldbplatform.tvos))(func)


def skipIfwatchOS(func):
    return skipIfPlatform(lldbplatform.translate(lldbplatform.watchos))(func)


def skipIfbridgeOS(func):
    return skipIfPlatform(lldbplatform.translate(lldbplatform.bridgeos))(func)


def skipIfDarwinEmbedded(func):
    """Decorate the item to skip tests that should be skipped on Darwin armv7/arm64 targets."""
    return skipIfPlatform(lldbplatform.translate(lldbplatform.darwin_embedded))(func)


def skipIfDarwinSimulator(func):
    """Decorate the item to skip tests that should be skipped on Darwin simulator targets."""
    return skipIfPlatform(lldbplatform.translate(lldbplatform.darwin_simulator))(func)


def skipIfFreeBSD(func):
    """Decorate the item to skip tests that should be skipped on FreeBSD."""
    return skipIfPlatform(["freebsd"])(func)


def skipIfNetBSD(func):
    """Decorate the item to skip tests that should be skipped on NetBSD."""
    return skipIfPlatform(["netbsd"])(func)


def skipIfDarwin(func):
    """Decorate the item to skip tests that should be skipped on Darwin."""
    return skipIfPlatform(lldbplatform.translate(lldbplatform.darwin_all))(func)


def skipIfLinux(func):
    """Decorate the item to skip tests that should be skipped on Linux."""
    return skipIfPlatform(["linux"])(func)


def skipIfWindows(func=None, windows_version=None):
    """Decorate the item to skip tests that should be skipped on Windows."""

    def decorator(func):
        if windows_version is None:
            return skipIfPlatform(["windows"])(func)
        else:
            actual_win_version = lldbplatformutil.getWindowsVersion()

            def version_check():
                if actual_win_version == "unknown":
                    return False
                operator, required_windows_version = windows_version
                return lldbplatformutil.isExpectedVersion(
                    actual_version=actual_win_version,
                    required_version=required_windows_version,
                    operator=operator,
                )

            return unittest.skipIf(
                version_check(),
                f"Test is skipped on Windows '{actual_win_version}'",
            )(func)

    if func is not None:
        return decorator(func)
    return decorator


def skipIfWindowsAndNonEnglish(func):
    """Decorate the item to skip tests that should be skipped on non-English locales on Windows."""

    def is_Windows_NonEnglish():
        if sys.platform != "win32":
            return None
        kernel = ctypes.windll.kernel32
        if locale.windows_locale[kernel.GetUserDefaultUILanguage()] == "en_US":
            return None
        return "skipping non-English Windows locale"

    return skipTestIfFn(is_Windows_NonEnglish)(func)


def skipUnlessWindows(func):
    """Decorate the item to skip tests that should be skipped on any non-Windows platform."""
    return skipUnlessPlatform(["windows"])(func)


def skipUnlessDarwin(func):
    """Decorate the item to skip tests that should be skipped on any non Darwin platform."""
    return skipUnlessPlatform(lldbplatformutil.getDarwinOSTriples())(func)


def skipUnlessTargetAndroid(func):
    return unittest.skipUnless(
        lldbplatformutil.target_is_android(), "requires target to be Android"
    )(func)


def skipIfHostIncompatibleWithTarget(func):
    """Decorate the item to skip tests when the host and target are incompatible."""

    def is_host_incompatible_with_remote():
        host_arch = lldbplatformutil.getLLDBArchitecture()
        host_platform = lldbplatformutil.getHostPlatform()
        target_arch = lldbplatformutil.getArchitecture()
        target_platform = lldbplatformutil.getPlatform()
        if (
            not (target_arch == "x86_64" and host_arch == "i386")
            and host_arch != target_arch
        ):
            return (
                "skipping because target %s is not compatible with host architecture %s"
                % (target_arch, host_arch)
            )
        if target_platform != host_platform:
            return "skipping because target is %s but host is %s" % (
                target_platform,
                host_platform,
            )
        if lldbplatformutil.match_android_device(target_arch):
            return "skipping because target is android"
        return None

    return skipTestIfFn(is_host_incompatible_with_remote)(func)


def skipIfPlatform(oslist):
    """Decorate the item to skip tests if running on one of the listed platforms."""
    # This decorator cannot be ported to `skipIf` yet because it is used on entire
    # classes, which `skipIf` explicitly forbids.
    return unittest.skipIf(
        lldbplatformutil.getPlatform() in oslist, "skip on %s" % (", ".join(oslist))
    )


def skipUnlessPlatform(oslist):
    """Decorate the item to skip tests unless running on one of the listed platforms."""
    # This decorator cannot be ported to `skipIf` yet because it is used on entire
    # classes, which `skipIf` explicitly forbids.
    return unittest.skipUnless(
        lldbplatformutil.getPlatform() in oslist,
        "requires one of %s" % (", ".join(oslist)),
    )


def skipUnlessArch(arch):
    """Decorate the item to skip tests unless running on the specified architecture."""

    def arch_doesnt_match():
        target_arch = lldbplatformutil.getArchitecture()
        if arch != target_arch:
            return "Test only runs on " + arch + ", but target arch is " + target_arch
        return None

    return skipTestIfFn(arch_doesnt_match)


def skipIfTargetAndroid(bugnumber=None, api_levels=None, archs=None):
    """Decorator to skip tests when the target is Android.

    Arguments:
        api_levels - The API levels for which the test should be skipped. If
            it is None, then the test will be skipped for all API levels.
        arch - A sequence of architecture names specifying the architectures
            for which a test is skipped. None means all architectures.
    """
    return skipTestIfFn(
        _skip_fn_for_android("skipping for android", api_levels, archs), bugnumber
    )


def skipUnlessAppleSilicon(func):
    """Decorate the item to skip tests unless running on Apple Silicon."""

    def not_apple_silicon():
        if platform.system() != "Darwin" or lldbplatformutil.getArchitecture() not in [
            "arm64",
            "arm64e",
        ]:
            return "Test only runs on Apple Silicon"
        return None

    return skipTestIfFn(not_apple_silicon)(func)


def skipUnlessSupportedTypeAttribute(attr):
    """Decorate the item to skip test unless Clang supports type __attribute__(attr)."""

    def compiler_doesnt_support_struct_attribute():
        compiler_path = lldbplatformutil.getCompiler()
        with temp_file.OnDiskTempFile() as f:
            cmd = [lldbplatformutil.getCompiler(), "-x", "c++", "-c", "-o", f.path, "-"]
            p = subprocess.Popen(
                cmd,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
            )
            stdout, stderr = p.communicate("struct __attribute__((%s)) Test {};" % attr)
            if attr in stderr:
                return "Compiler does not support attribute %s" % (attr)
            return None

    return skipTestIfFn(compiler_doesnt_support_struct_attribute)


def skipUnlessHasCallSiteInfo(func):
    """Decorate the function to skip testing unless call site info from clang is available."""

    def is_compiler_clang_with_call_site_info():
        compiler_path = lldbplatformutil.getCompiler()
        compiler = os.path.basename(compiler_path)
        if not compiler.startswith("clang"):
            return "Test requires clang as compiler"

        with temp_file.OnDiskTempFile() as f:
            cmd = (
                "echo 'int main() {}' | "
                "%s -g -glldb -O1 -S -emit-llvm -x c -o %s -" % (compiler_path, f.path)
            )
            if os.popen(cmd).close() is not None:
                return "Compiler can't compile with call site info enabled"

            with open(f.path, "r") as ir_output_file:
                buf = ir_output_file.read()

            if "DIFlagAllCallsDescribed" not in buf:
                return "Compiler did not introduce DIFlagAllCallsDescribed IR flag"

            return None

    return skipTestIfFn(is_compiler_clang_with_call_site_info)(func)


def skipUnlessCompilerIsClang(func):
    """Decorate the item to skip test unless the compiler is clang."""

    def is_compiler_clang():
        compiler_path = lldbplatformutil.getCompiler()
        compiler = os.path.basename(compiler_path)
        if not compiler.startswith("clang"):
            return "Test requires clang as compiler"
        return None

    return skipTestIfFn(is_compiler_clang)(func)


def skipUnlessMSVC(func):
    """Decorate the item to skip test unless msvc is available."""

    def is_msvc_in_path():
        result = subprocess.run(
            ["cl.exe"],
            capture_output=True,
            text=True,
        )
        if result.returncode != 0:
            return f"Test requires MSVC to be in the Path."
        return None

    return skipTestIfFn(is_msvc_in_path)(func)


def skipUnlessThreadSanitizer(func):
    """Decorate the item to skip test unless Clang -fsanitize=thread is supported."""

    def is_compiler_clang_with_thread_sanitizer():
        if is_running_under_asan():
            return "Thread sanitizer tests are disabled when runing under ASAN"

        compiler_path = lldbplatformutil.getCompiler()
        compiler = os.path.basename(compiler_path)
        if not compiler.startswith("clang"):
            return "Test requires clang as compiler"
        if lldbplatformutil.getPlatform() == "windows":
            return "TSAN tests not compatible with 'windows'"
        # rdar://28659145 - TSAN tests don't look like they're supported on i386
        if (
            lldbplatformutil.getArchitecture() == "i386"
            and platform.system() == "Darwin"
        ):
            return "TSAN tests not compatible with i386 targets"
        if not _compiler_supports(compiler_path, "-fsanitize=thread"):
            return "Compiler cannot compile with -fsanitize=thread"
        return None

    return skipTestIfFn(is_compiler_clang_with_thread_sanitizer)(func)


def skipUnlessUndefinedBehaviorSanitizer(func):
    """Decorate the item to skip test unless -fsanitize=undefined is supported."""

    def is_compiler_clang_with_ubsan():
        if is_running_under_asan():
            return (
                "Undefined behavior sanitizer tests are disabled when runing under ASAN"
            )

        # We need to write out the object into a named temp file for inspection.
        with temp_file.OnDiskTempFile() as outputf:
            # Try to compile with ubsan turned on.
            if not _compiler_supports(
                lldbplatformutil.getCompiler(),
                "-fsanitize=undefined",
                "int main() { int x = 0; return x / x; }",
                outputf,
            ):
                return "Compiler cannot compile with -fsanitize=undefined"
            if not outputf.path:
                return "Cannot create Temp file path."

            nm_bin = configuration.get_nm_path()
            if not nm_bin:
                return "No llvm-nm or nm binary."

            # Check that we actually see ubsan instrumentation in the binary.
            nm_output = subprocess.check_output([nm_bin, outputf.path], text=True)
            if "__ubsan_handle_divrem_overflow" not in nm_output:
                return "Division by zero instrumentation is missing"

        return None

    return skipTestIfFn(is_compiler_clang_with_ubsan)(func)


def is_running_under_asan():
    if "ASAN_OPTIONS" in os.environ:
        return "ASAN unsupported"
    return None


def is_running_under_mte():
    if configuration.mte_enabled:
        return "MTE unsupported"
    return None


def skipIfMTE(func):
    """Skip this test when running with MTE (Memory Tagging Extension) enabled."""
    return skipTestIfFn(is_running_under_mte)(func)


def skipUnlessAddressSanitizer(func):
    """Decorate the item to skip test unless Clang -fsanitize=thread is supported."""

    def is_compiler_with_address_sanitizer():
        # Also don't run tests that use address sanitizer inside an
        # address-sanitized LLDB. The tests don't support that
        # configuration.
        if is_running_under_asan():
            return "Address sanitizer tests are disabled when runing under ASAN"

        if lldbplatformutil.getPlatform() == "windows":
            return "ASAN tests not compatible with 'windows'"
        if not _compiler_supports(lldbplatformutil.getCompiler(), "-fsanitize=address"):
            return "Compiler cannot compile with -fsanitize=address"
        return None

    return skipTestIfFn(is_compiler_with_address_sanitizer)(func)


def skipUnlessBoundsSafety(func):
    """Decorate the item to skip test unless Clang -fbounds-safety is supported."""

    def is_compiler_with_bounds_safety():
        if not _compiler_supports(lldbplatformutil.getCompiler(), "-fbounds-safety"):
            return "Compiler cannot compile with -fbounds-safety"
        return None

    return skipTestIfFn(is_compiler_with_bounds_safety)(func)


def skipUnlessCompilerSupports(flag):
    """Decorate the item to skip the test unless the compiler supports this flag."""

    def does_compiler_support_flag():
        if not _compiler_supports(lldbplatformutil.getCompiler(), flag):
            return f"Compiler does not support flag {flag}"
        return None

    return skipTestIfFn(does_compiler_support_flag)


def skipIfAsan(func):
    """Skip this test if the environment is set up to run LLDB *itself* under ASAN."""
    return skipTestIfFn(is_running_under_asan)(func)


def skipUnlessAArch64MTELinuxCompiler(func):
    """Decorate the item to skip test unless MTE is supported by the test compiler."""

    def is_toolchain_with_mte():
        compiler_path = lldbplatformutil.getCompiler()
        with temp_file.OnDiskTempFile() as f:
            if lldbplatformutil.getPlatform() == "windows":
                return "MTE tests are not compatible with 'windows'"

            cmd = f"{compiler_path} -x c -o {f.path} -"
            if (
                subprocess.run(
                    cmd, shell=True, input="int main() {}".encode()
                ).returncode
                != 0
            ):
                # Cannot compile at all, don't skip the test
                # so that we report the broken compiler normally.
                return None

            # We need the Linux headers and ACLE MTE intrinsics
            test_src = """
                #include <asm/hwcap.h>
                #include <arm_acle.h>
                #ifndef HWCAP2_MTE
                #error
                #endif
                int main() {
                    void* ptr = __arm_mte_create_random_tag((void*)(0), 0);
                }"""
            cmd = f"{compiler_path} -march=armv8.5-a+memtag -x c -o {f.path} -"
            res = subprocess.run(cmd, shell=True, input=test_src.encode())
            if res.returncode != 0:
                return "Toolchain does not support MTE"
            return None

    return skipTestIfFn(is_toolchain_with_mte)(func)


def _get_bool_config(key, fail_value=True):
    """
    Returns the current LLDB's build config value.
    :param key The key to lookup in LLDB's build configuration.
    :param fail_value The error value to return when the key can't be found.
           Defaults to true so that if an unknown key is lookup up we rather
           enable more tests (that then fail) than silently skipping them.
    """
    config = lldb.SBDebugger.GetBuildConfiguration()
    value_node = config.GetValueForKey(key)
    return value_node.GetValueForKey("value").GetBooleanValue(fail_value)


def _get_bool_config_skip_if_decorator(key):
    have = _get_bool_config(key)
    return unittest.skipIf(not have, "requires " + key)


def skipIfCurlSupportMissing(func):
    return _get_bool_config_skip_if_decorator("curl")(func)


def skipIfCursesSupportMissing(func):
    return _get_bool_config_skip_if_decorator("curses")(func)


def skipIfXmlSupportMissing(func):
    return _get_bool_config_skip_if_decorator("xml")(func)


def skipIfEditlineSupportMissing(func):
    return _get_bool_config_skip_if_decorator("editline")(func)


def skipIfEditlineWideCharSupportMissing(func):
    return _get_bool_config_skip_if_decorator("editline_wchar")(func)


def skipIfZLIBSupportMissing(func):
    return _get_bool_config_skip_if_decorator("zlib")(func)


def skipIfLLVMTargetMissing(target):
    config = lldb.SBDebugger.GetBuildConfiguration()
    targets = config.GetValueForKey("targets").GetValueForKey("value")
    found = False
    for i in range(targets.GetSize()):
        if targets.GetItemAtIndex(i).GetStringValue(99) == target:
            found = True
            break

    return unittest.skipIf(not found, "requires " + target)


def skipUnlessFeature(cpu_feature: CPUFeature):
    def hasFeature(test_case):
        if not test_case.isSupported(cpu_feature):
            return f"Unsupported CPU feature: {cpu_feature}"
        return None

    return skipTestIfFn(hasFeature)


def skipIfBuildType(types: list[str]):
    """Skip tests if built in a specific CMAKE_BUILD_TYPE.

    Supported types include 'Release', 'RelWithDebInfo', 'Debug', 'MinSizeRel'.
    """
    types = [name.lower() for name in types]
    return unittest.skipIf(
        configuration.cmake_build_type is not None
        and configuration.cmake_build_type.lower() in types,
        "skip on {} build type(s)".format(", ".join(types)),
    )


def skipUnlessArm64eSupported(func):
    """Decorate the item to skip test unless Clang can target arm64e."""

    def can_build_and_run_arm64e():
        arch = lldbplatformutil.getArchitecture()

        # We need to be running the test suite for arm64 or arm64e. If we're
        # running the whole test suite as arm64e, we don't need any additional
        # checks.
        if arch == "arm64e":
            return None
        elif arch != "arm64":
            return "Not targeting arm64"

        # Need at least macOS Tahoe (26) to run arm64e binaries.
        if platform.mac_ver()[0] == "" or _check_expected_version(
            "<", "26.0", platform.mac_ver()[0]
        ):
            return "Host cannot run arm64e binaries"

        # Need a compiler that can target arm64e.
        compiler_path = lldbplatformutil.getCompiler()
        if not _compiler_supports(compiler_path, "-arch arm64e"):
            return "Compiler cannot target arm64e"

        # Need debugserver built with arm64e support.
        if not configuration.arm64e_debugserver:
            return "debugserver not built with arm64e support"

        # Technically ASan is supported, but we need an arm64e sanitizer
        # runtime and we assume that's not the case unless we run the whole
        # test suite as arm64e.
        if is_running_under_asan():
            return "Sanitizer runtime may not support arm64e"

        return None

    return skipTestIfFn(can_build_and_run_arm64e)(func)



import lldb
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestTraceDumpInfo(TraceIntelPTTestCaseBase):
    def testErrorMessages(self):
        # We first check the output when there are no targets
        self.expect(
            "thread trace dump info",
            substrs=[
                "error: invalid target, create a target using the 'target create' command"
            ],
            error=True,
        )

        # We now check the output when there's a non-running target
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )

        self.expect(
            "thread trace dump info",
            substrs=["error: Command requires a current process."],
            error=True,
        )

        # Now we check the output when there's a running target without a trace
        self.expect("b main")
        self.expect("run")

        self.expect(
            "thread trace dump info",
            substrs=["error: Process is not being traced"],
            error=True,
        )

    def testDumpRawTraceSize(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        self.expect(
            "thread trace dump info",
            substrs=[
                """thread #1: tid = 3842849

  Trace technology: intel-pt

  Total number of trace items: 28

  Memory usage:
    Raw trace size: 4 KiB""",
                """

  Events:
    Number of individual events: 7
      software disabled tracing: 2
      hardware disabled tracing: 4
      trace synchronization point: 1""",
            ],
            patterns=[r"Decoding instructions: \d.\d\ds"],
        )

    def testDumpRawTraceSizeJSON(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        self.expect(
            "thread trace dump info --json ",
            substrs=[
                """{
  "traceTechnology": "intel-pt",
  "threadStats": {
    "tid": 3842849,
    "traceItemsCount": 28,""",
                """
    },
    "events": {
      "totalCount": 7,
      "individualCounts": {
        "software disabled tracing": 2,
        "hardware disabled tracing": 4,
        "trace synchronization point": 1
      }
    },
    "errors": {
      "totalCount": 0,
      "libiptErrors": {},
      "fatalErrors": 0,
      "otherErrors": 0
    }
  },
  "globalStats": {
    "timingInSeconds": {}
  }
}""",
            ],
        )



"""
Test address breakpoints set with shared library of SBAddress work correctly.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class AddressBreakpointTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_address_breakpoints(self):
        """Test address breakpoints set with shared library of SBAddress work correctly."""
        self.build()
        self.address_breakpoints()

    def address_breakpoints(self):
        """Test address breakpoints set with shared library of SBAddress work correctly."""
        target = self.createTestTarget()

        # Now create a breakpoint on main.c by name 'c'.
        breakpoint = target.BreakpointCreateBySourceRegex(
            "Set a breakpoint here", lldb.SBFileSpec("main.c")
        )
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() >= 1, VALID_BREAKPOINT
        )

        # Get the breakpoint location from breakpoint after we verified that,
        # indeed, it has one location.
        location = breakpoint.GetLocationAtIndex(0)
        self.assertTrue(location and location.IsEnabled(), VALID_BREAKPOINT_LOCATION)

        # Next get the address from the location, and create an address breakpoint using
        # that address:

        address = location.GetAddress()
        target.BreakpointDelete(breakpoint.GetID())

        breakpoint = target.BreakpointCreateBySBAddress(address)

        # Disable ASLR.  This will allow us to actually test (on platforms that support this flag)
        # that the breakpoint was able to track the module.

        launch_info = lldb.SBLaunchInfo(None)
        flags = launch_info.GetLaunchFlags()
        flags &= ~lldb.eLaunchFlagDisableASLR
        flags &= lldb.eLaunchFlagInheritTCCFromParent
        launch_info.SetLaunchFlags(flags)

        error = lldb.SBError()

        process = target.Launch(launch_info, error)
        self.assertTrue(process, PROCESS_IS_VALID)

        # Did we hit our breakpoint?
        from lldbsuite.test.lldbutil import get_threads_stopped_at_breakpoint

        threads = get_threads_stopped_at_breakpoint(process, breakpoint)
        self.assertEqual(
            len(threads), 1, "There should be a thread stopped at our breakpoint"
        )

        # The hit count for the breakpoint should be 1.
        self.assertEqual(breakpoint.GetHitCount(), 1)

        process.Kill()

        # Now re-launch and see that we hit the breakpoint again:
        launch_info.Clear()
        launch_info.SetLaunchFlags(flags)

        process = target.Launch(launch_info, error)
        self.assertTrue(process, PROCESS_IS_VALID)

        thread = get_threads_stopped_at_breakpoint(process, breakpoint)
        self.assertEqual(
            len(threads), 1, "There should be a thread stopped at our breakpoint"
        )

        # The hit count for the breakpoint should be 1, since we reset counts
        # for each run.
        self.assertEqual(breakpoint.GetHitCount(), 1)



"""
Test that inlined breakpoints (breakpoint set on a file/line included from
another source file) works correctly.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class InlinedBreakpointsTestCase(TestBase):
    """Bug fixed: rdar://problem/8464339"""

    def test_with_run_command(self):
        """Test 'b basic_types.cpp:176' does break (where int.cpp includes basic_type.cpp)."""
        self.build()
        self.inlined_breakpoints()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside basic_type.cpp.
        self.line = line_number("basic_type.cpp", "// Set break point at this line.")

    def inlined_breakpoints(self):
        """Test 'b basic_types.cpp:176' does break (where int.cpp includes basic_type.cpp)."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # With the inline-breakpoint-strategy, our file+line breakpoint should
        # not resolve to a location.
        self.runCmd("settings set target.inline-breakpoint-strategy headers")

        # Set a breakpoint and fail because it is in an inlined source
        # implemenation file
        lldbutil.run_break_set_by_file_and_line(
            self, "basic_type.cpp", self.line, num_expected_locations=0
        )

        # Now enable breakpoints in implementation files and see the breakpoint
        # set succeed
        self.runCmd("settings set target.inline-breakpoint-strategy always")
        # And add hooks to restore the settings during tearDown().
        self.addTearDownHook(
            lambda: self.runCmd("settings set target.inline-breakpoint-strategy always")
        )

        lldbutil.run_break_set_by_file_and_line(
            self, "basic_type.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        # And it should break at basic_type.cpp:176.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=[
                "stopped",
                "basic_type.cpp:%d" % self.line,
                "stop reason = breakpoint",
            ],
        )



"""
Test that the the expression parser enables ObjC support
when stopped in a C++ frame without debug-info.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCFromCppFramesWithoutDebugInfo(TestBase):
    def test(self):
        self.build()
        (_, process, _, _) = lldbutil.run_to_name_breakpoint(self, "main")

        self.assertState(process.GetState(), lldb.eStateStopped)

        # Tests that we can use builtin Objective-C identifiers.
        self.expect("expr id", error=False)

        # Tests that we can lookup Objective-C decls in the ObjC runtime plugin.
        self.expect_expr(
            "NSString *c; c == nullptr", result_value="true", result_type="bool"
        )



"""
Test lldb-dap attach request
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os
import subprocess
import threading
import time


# Often fails on Arm Linux, but not specifically because it's Arm, something in
# process scheduling can cause a massive (minutes) delay during this test.
@skipIf(oslist=["linux"], archs=["arm$"])
class TestDAP_attach(lldbdap_testcase.DAPTestCaseBase):
    SHARED_BUILD_TESTCASE = False

    def spawn(self, program, args=None):
        return self.spawnSubprocess(
            executable=program,
            args=args,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
        )

    def spawn_and_wait(self, program, delay):
        time.sleep(delay)
        proc = self.spawn(program=program)
        start_time = time.time()
        # Wait for either the process to exit or the event to be set.
        while proc.poll() is None and not self.spawn_event.is_set():
            elapsed = time.time() - start_time
            if elapsed >= self.DEFAULT_TIMEOUT:
                break
            time.sleep(0.1)
        proc.kill()
        proc.wait()

    def continue_and_verify_pid(self):
        self.do_continue()
        proc = self.lastSubprocess
        if proc is None:
            self.fail(f"lastSubprocess is None")
        out, _ = proc.communicate("foo")
        self.assertIn(f"pid = {proc.pid}", out)

    def test_by_pid(self):
        """
        Tests attaching to a process by process ID.
        """
        program = self.build_and_create_debug_adapter_for_attach()
        proc = self.spawn(program=program)
        self.attach(pid=proc.pid)
        self.continue_and_verify_pid()

    def test_by_name(self):
        """
        Tests attaching to a process by process name.
        """
        program = self.build_and_create_debug_adapter_for_attach()

        # Use a file as a synchronization point between test and inferior.
        pid_file_path = lldbutil.append_to_process_working_directory(
            self, "pid_file_%d" % (int(time.time()))
        )
        self.spawn(program=program, args=[pid_file_path])
        lldbutil.wait_for_file_on_target(self, pid_file_path)

        self.attach(program=program)
        self.continue_and_verify_pid()

    @expectedFailureWindows
    def test_by_name_waitFor(self):
        """
        Tests waiting for, and attaching to a process by process name that
        doesn't exist yet.
        """
        program = self.build_and_create_debug_adapter_for_attach()
        self.spawn_event = threading.Event()
        self.spawn_thread = threading.Thread(
            target=self.spawn_and_wait,
            args=(
                program,
                1.0,
            ),
        )
        self.spawn_thread.start()
        try:
            self.attach(program=program, waitFor=True)
            self.continue_and_verify_pid()
        finally:
            self.spawn_event.set()
            if self.spawn_thread.is_alive():
                self.spawn_thread.join(timeout=10)

    @expectedFailureWindows
    def test_by_partial_name_waitFor(self):
        """
        Tests waiting for and attaching to a process by partial process name
        that doesn't exist yet.
        """
        program = self.build_and_create_debug_adapter_for_attach()
        self.spawn_event = threading.Event()
        self.spawn_thread = threading.Thread(
            target=self.spawn_and_wait,
            args=(
                program,
                1.0,
            ),
        )
        self.spawn_thread.start()
        try:
            self.attach(program=os.path.basename(program), waitFor=True)
            self.continue_and_verify_pid()
        finally:
            self.spawn_event.set()
            if self.spawn_thread.is_alive():
                self.spawn_thread.join(timeout=10)

    def test_attach_with_missing_session_debugger(self):
        """
        Test that attaching with only one of debuggerId/targetId specified
        fails with the expected error message.
        """
        self.build_and_create_debug_adapter()

        # Test with only targetId specified (no debuggerId)
        session = {"targetId": 99999}
        attach_seq = self.attach(session=session)
        resp = self.dap_server.receive_response(attach_seq)
        self.assertFalse(resp["success"])
        self.assertIn(
            "missing value at arguments.session.debuggerId",
            resp["body"]["error"]["format"],
        )

    def test_attach_with_invalid_session(self):
        """
        Test that attaching with both debuggerId and targetId specified but
        invalid fails with an appropriate error message.
        """
        self.build_and_create_debug_adapter()

        # Attach with both debuggerId=9999 and targetId=99999 (both invalid).
        # Since debugger ID 9999 likely doesn't exist in the global registry,
        # we expect a validation error.
        session = {"debuggerId": 9999, "targetId": 9999}
        resp = self.attach_and_configurationDone(session=session)
        self.assertFalse(resp["success"])
        error_msg = resp["body"]["error"]["format"]
        # Either error is acceptable - both indicate the debugger reuse
        # validation is working correctly
        self.assertTrue(
            "Unable to find existing debugger" in error_msg
            or f"Expected debugger/target not found error, got: {error_msg}"
        )



"""
Test that memory tagging features work with Linux core files.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class AArch64LinuxMTEMemoryTagCoreFileTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    MTE_BUF_ADDR = hex(0xFFFFA733B000)
    BUF_ADDR = hex(0xFFFFA733A000)

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_tag_core_file_memory_region(self):
        """Test that memory regions are marked as tagged when there is a tag
        segment in the core file."""
        self.runCmd("target create --core core.mte")

        # There should only be one tagged region.
        self.runCmd("memory region --all")
        got = self.res.GetOutput()
        found_tagged_region = False

        for line in got.splitlines():
            if "memory tagging: enabled" in line:
                if found_tagged_region:
                    self.fail("Expected only one tagged region.")
                found_tagged_region = True

        self.assertTrue(found_tagged_region, "Did not find a tagged memory region.")

        # mte_buf is tagged, buf is not.
        tagged = "memory tagging: enabled"
        self.expect("memory region {}".format(self.MTE_BUF_ADDR), patterns=[tagged])
        self.expect(
            "memory region {}".format(self.BUF_ADDR), patterns=[tagged], matching=False
        )

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_tag_core_file_tag_write(self):
        """Test that "memory tag write" does not work with core files
        as they are read only."""
        self.runCmd("target create --core core.mte")

        self.expect(
            "memory tag write {} 1".format(self.MTE_BUF_ADDR),
            error=True,
            patterns=["error: elf-core does not support writing memory tags"],
        )

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_tag_core_file_tag_read(self):
        """Test that "memory tag read" works with core files."""
        self.runCmd("target create --core core.mte")

        # Tags are packed 2 per byte meaning that in addition to granule alignment
        # there is also 2 x granule alignment going on.

        # All input validation should work as normal.
        not_tagged_pattern = (
            "error: Address range 0x[A-Fa-f0-9]+:0x[A-Fa-f0-9]+ "
            "is not in a memory tagged region"
        )
        self.expect(
            "memory tag read {}".format(self.BUF_ADDR),
            error=True,
            patterns=[not_tagged_pattern],
        )
        # The first part of this range is not tagged.
        self.expect(
            "memory tag read {addr}-16 {addr}+16".format(addr=self.MTE_BUF_ADDR),
            error=True,
            patterns=[not_tagged_pattern],
        )
        # The last part of this range is not tagged.
        self.expect(
            "memory tag read {addr}+4096-16 {addr}+4096+16".format(
                addr=self.MTE_BUF_ADDR
            ),
            error=True,
            patterns=[not_tagged_pattern],
        )

        self.expect(
            "memory tag read {addr}+16 {addr}".format(addr=self.MTE_BUF_ADDR),
            error=True,
            patterns=[
                r"error: End address \(0x[A-Fa-f0-9]+\) "
                "must be greater than the start address "
                r"\(0x[A-Fa-f0-9]+\)"
            ],
        )

        # The simplest scenario. 2 granules means 1 byte of packed tags
        # with no realignment required.
        self.expect(
            "memory tag read {addr} {addr}+32".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                "\\[0x[A-Fa-f0-9]+00, 0x[A-Fa-f0-9]+10\\): 0x0\n"
                "\\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\\): 0x1 \\(mismatch\\)$"
            ],
        )

        # Here we want just one tag so must use half of the first byte.
        # (start is aligned length is not)
        self.expect(
            "memory tag read {addr} {addr}+16".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n" r"\[0x[A-Fa-f0-9]+00, 0x[A-Fa-f0-9]+10\): 0x0$"
            ],
        )
        # Get the other half of the first byte.
        # (end is aligned start is not)
        self.expect(
            "memory tag read {addr}+16 {addr}+32".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                r"\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\): 0x1 \(mismatch\)$"
            ],
        )

        # Same thing but with a starting range > 1 granule.
        self.expect(
            "memory tag read {addr} {addr}+48".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                "\\[0x[A-Fa-f0-9]+00, 0x[A-Fa-f0-9]+10\\): 0x0\n"
                "\\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\\): 0x1 \\(mismatch\\)\n"
                "\\[0x[A-Fa-f0-9]+20, 0x[A-Fa-f0-9]+30\\): 0x2 \\(mismatch\\)$"
            ],
        )
        self.expect(
            "memory tag read {addr}+16 {addr}+64".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                "\\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\\): 0x1 \\(mismatch\\)\n"
                "\\[0x[A-Fa-f0-9]+20, 0x[A-Fa-f0-9]+30\\): 0x2 \\(mismatch\\)\n"
                "\\[0x[A-Fa-f0-9]+30, 0x[A-Fa-f0-9]+40\\): 0x3 \\(mismatch\\)$"
            ],
        )
        # Here both start and end are unaligned.
        self.expect(
            "memory tag read {addr}+16 {addr}+80".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                "\\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\\): 0x1 \\(mismatch\\)\n"
                "\\[0x[A-Fa-f0-9]+20, 0x[A-Fa-f0-9]+30\\): 0x2 \\(mismatch\\)\n"
                "\\[0x[A-Fa-f0-9]+30, 0x[A-Fa-f0-9]+40\\): 0x3 \\(mismatch\\)\n"
                "\\[0x[A-Fa-f0-9]+40, 0x[A-Fa-f0-9]+50\\): 0x4 \\(mismatch\\)$"
            ],
        )

        # For the initial alignment of start/end to granule boundaries the tag manager
        # is used, so this reads 1 tag as it would normally.
        self.expect(
            "memory tag read {addr} {addr}+1".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n" r"\[0x[A-Fa-f0-9]+00, 0x[A-Fa-f0-9]+10\): 0x0$"
            ],
        )

        # This range is aligned to granules as mte_buf to mte_buf+32 so the result
        # should be 2 granules.
        self.expect(
            "memory tag read {addr} {addr}+17".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                "\\[0x[A-Fa-f0-9]+00, 0x[A-Fa-f0-9]+10\\): 0x0\n"
                "\\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\\): 0x1 \\(mismatch\\)$"
            ],
        )

        # Alignment of this range causes it to become unaligned to 2*granule boundaries.
        self.expect(
            "memory tag read {addr} {addr}+33".format(addr=self.MTE_BUF_ADDR),
            patterns=[
                "Allocation tags:\n"
                "\\[0x[A-Fa-f0-9]+00, 0x[A-Fa-f0-9]+10\\): 0x0\n"
                "\\[0x[A-Fa-f0-9]+10, 0x[A-Fa-f0-9]+20\\): 0x1 \\(mismatch\\)\n",
                "\\[0x[A-Fa-f0-9]+20, 0x[A-Fa-f0-9]+30\\): 0x2 \\(mismatch\\)$",
            ],
        )

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_commands_no_mte(self):
        """Test that memory tagging commands fail on an AArch64 corefile without
        any tag segments."""

        self.runCmd("target create --core core.nomte")

        self.expect(
            "memory tag read 0 1",
            substrs=["error: Process does not support memory tagging"],
            error=True,
        )
        # Note that this tells you memory tagging is not supported at all, versus
        # the MTE core file which does support it but does not allow writing tags.
        self.expect(
            "memory tag write 0 1",
            substrs=["error: Process does not support memory tagging"],
            error=True,
        )

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_tag_fault_reason(self):
        """Test that we correctly report the fault reason."""
        self.runCmd("target create --core core.mte")

        # There is no fault address shown here because core files do not include
        # si_addr.
        self.expect(
            "bt",
            substrs=[
                "* thread #1, name = 'a.out.mte', stop reason = SIGSEGV: sync tag check fault (fault address=0xffffa733b010)"
            ],
        )

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_ctrl_register(self):
        """Test that we correctly report the mte_ctrl register"."""
        # The register is present even if MTE is not used in the current process
        # and also on targets without MTE, because it controls parts of the
        # overall tagged address ABI as well.
        self.runCmd("target create --core core.nomte")
        self.expect("register read mte_ctrl", substrs=["mte_ctrl = 0x0000000000000000"])

        self.runCmd("target create --core core.mte")
        # The expected value is:
        # * Allowed tags value of 0xFFFF, shifted up by 3 resulting in 0x7fff8.
        # * Bit 19 set to 0, which means that store only checking is disabled.
        # * Bit 1 set to enable synchronous tag faults.
        # * Bit 0 set to enable the tagged address ABI.
        expected = ["mte_ctrl = 0x000000000007fffb"]

        if self.hasXMLSupport():
            expected.append(
                "(STORE_ONLY = 0, TAGS = 65535, TCF = TCF_SYNC, TAGGED_ADDR_ENABLE = 1)"
            )

        self.expect("register read mte_ctrl", substrs=expected)

        if self.hasXMLSupport():
            # Should get enumerator descriptions for TCF
            self.expect(
                "register info mte_ctrl",
                substrs=[
                    "TCF: 0 = TCF_NONE, 1 = TCF_SYNC, 2 = TCF_ASYNC, 3 = TCF_ASYMM"
                ],
            )

    @skipIfLLVMTargetMissing("AArch64")
    def test_mte_no_tags(self):
        """Test that we handle there being a tag segment but that segment does
        not contain any tag data. This can happen when the core is dumped
        with a restrictive limit or filter."""
        self.runCmd("target create --core core.mte.notags")

        mte_buf_addr = 0xFFFFA4AF3000

        # We can see which memory was tagged.
        self.expect(
            f"memory region {mte_buf_addr}", substrs=["memory tagging: enabled"]
        )

        # We cannot read those tags.
        self.expect(
            f"memory tag read {mte_buf_addr}",
            substrs=[
                "Could not read tags from core file segment. Segment is missing some or all tag data."
            ],
            error=True,
        )



"""
Test breakpoint command for different options.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class BreakpointOptionsTestCase(TestBase):
    def test(self):
        """Test breakpoint command for different options."""
        self.build()
        self.breakpoint_options_test()
        self.breakpoint_options_language_test()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def breakpoint_options_test(self):
        """Test breakpoint command for different options."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint with 1 locations.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, extra_options="-K 1", num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, extra_options="-K 0", num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Stopped once.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 2."],
        )

        # Check the list of breakpoint.
        self.expect(
            "breakpoint list -f",
            "Breakpoint locations shown correctly",
            substrs=[
                "1: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.line,
                "2: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.line,
            ],
        )

        # Continue the program, there should be another stop.
        self.runCmd("process continue")

        # Stopped again.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        # Continue the program, we should exit.
        self.runCmd("process continue")

        # We should exit.
        self.expect(
            "process status",
            "Process exited successfully",
            patterns=["^Process [0-9]+ exited with status = 0"],
        )

    def breakpoint_options_language_test(self):
        """Test breakpoint command for language option."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint with 1 locations.
        bp_id = lldbutil.run_break_set_by_symbol(
            self,
            "ns::func",
            sym_exact=False,
            extra_options="-L c++",
            num_expected_locations=1,
        )

        location = self.target().FindBreakpointByID(bp_id).GetLocationAtIndex(0)
        function = location.GetAddress().GetFunction()
        self.expect(
            "breakpoint list -v",
            "Verbose breakpoint list contains mangled names",
            # The demangled function name is system-dependent, e.g.
            # 'int ns::func(void)' on Windows and 'ns::func()' on Linux.
            substrs=[
                f"function = {function.GetName()}",
                f"mangled function = {function.GetMangledName()}",
            ],
        )

        # This should create a breakpoint with 0 locations.
        lldbutil.run_break_set_by_symbol(
            self,
            "ns::func",
            sym_exact=False,
            extra_options="-L c",
            num_expected_locations=0,
        )
        self.runCmd("settings set target.language c")
        lldbutil.run_break_set_by_symbol(
            self, "ns::func", sym_exact=False, num_expected_locations=0
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Stopped once.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        # Continue the program, we should exit.
        self.runCmd("process continue")

        # We should exit.
        self.expect(
            "process status",
            "Process exited successfully",
            patterns=["^Process [0-9]+ exited with status = 0"],
        )



#!/usr/bin/env python3

import argparse
import re
import os

HEADER = """\
//===-- SBLanguages.h -----------------------------------------*- C++ -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

#ifndef LLDB_API_SBLANGUAGE_H
#define LLDB_API_SBLANGUAGE_H

#include <cstdint>

namespace lldb {
/// Used by \\ref SBExpressionOptions.
/// These enumerations use the same language enumerations as the DWARF
/// specification for ease of use and consistency.
enum SBSourceLanguageName : uint16_t {
"""

FOOTER = """\
};

} // namespace lldb

#endif
"""

REGEX = re.compile(
    r'^ *HANDLE_DW_LNAME *\( *(?P<value>[^,]+), (?P<name>.*), "(?P<comment>[^"]+)",.*\)'
)


def emit_enum(input, output):
    # Read the input and break it up by lines.
    lines = []
    with open(input, "r") as f:
        lines = f.readlines()

    # Create output folder if it does not exist
    os.makedirs(os.path.dirname(output), exist_ok=True)

    # Write the output.
    with open(output, "w") as f:
        # Emit the header.
        f.write(HEADER)

        # Emit the enum values.
        for line in lines:
            match = REGEX.match(line)
            if not match:
                continue
            f.write(f"  /// {match.group('comment')}.\n")
            f.write(f"  eLanguageName{match.group('name')} = {match.group('value')},\n")

        # Emit the footer
        f.write(FOOTER)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--output", "-o")
    parser.add_argument("input")
    args = parser.parse_args()

    emit_enum(args.input, args.output)


if __name__ == "__main__":
    main()



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDataFormatterGenericForwardList(TestBase):
    SHARED_BUILD_TESTCASE = False
    TEST_WITH_PDB_DEBUG_INFO = True

    def setUp(self):
        TestBase.setUp(self)
        self.line = line_number("main.cpp", "// break here")
        self.namespace = "std"

    def do_test(self):
        """Test that std::forward_list is displayed correctly"""
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        forward_list = self.namespace + "::forward_list"
        self.expect("frame variable empty", substrs=[forward_list, "size=0", "{}"])

        self.expect(
            "frame variable one_elt",
            substrs=[forward_list, "size=1", "{", "[0] = 47", "}"],
        )

        self.expect(
            "frame variable five_elts",
            substrs=[
                forward_list,
                "size=5",
                "{",
                "[0] = 1",
                "[1] = 22",
                "[2] = 333",
                "[3] = 4444",
                "[4] = 55555",
                "}",
            ],
        )

        self.expect(
            "settings show target.max-children-count",
            matching=True,
            substrs=["target.max-children-count (unsigned) = 24"],
        )

        self.expect(
            "frame variable thousand_elts",
            matching=False,
            substrs=["[256]", "[333]", "[444]", "[555]", "[666]", "..."],
        )
        self.runCmd("settings set target.max-children-count 3", check=False)

        self.expect(
            "frame variable thousand_elts",
            matching=False,
            substrs=[
                "[3]",
                "[4]",
                "[5]",
            ],
        )

        self.expect(
            "frame variable thousand_elts",
            matching=True,
            substrs=["size=24", "[0]", "[1]", "[2]", "..."],
        )

    def do_test_ptr_and_ref(self):
        """Test that ref and ptr to std::forward_list is displayed correctly"""
        (_, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Check ref and ptr", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect("frame variable ref", substrs=["size=0", "{}"])

        self.expect("frame variable *ptr", substrs=["size=0", "{}"])

        lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect("frame variable ref", substrs=["{", "[0] = 47", "}"])

        self.expect("frame variable *ptr", substrs=["{", "[0] = 47", "}"])

        lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect(
            "frame variable ref",
            substrs=[
                "size=5",
                "{",
                "[0] = 1",
                "[1] = 22",
                "[2] = 333",
                "[3] = 4444",
                "[4] = 55555",
                "}",
            ],
        )

        self.expect(
            "frame variable *ptr",
            substrs=[
                "size=5",
                "{",
                "[0] = 1",
                "[1] = 22",
                "[2] = 333",
                "[3] = 4444",
                "[4] = 55555",
                "}",
            ],
        )

        lldbutil.continue_to_breakpoint(process, bkpt)

        self.runCmd("settings set target.max-children-count 256", check=False)

        self.expect(
            "settings show target.max-children-count",
            matching=True,
            substrs=["target.max-children-count (unsigned) = 256"],
        )

        self.expect(
            "frame variable ref",
            matching=True,
            substrs=[
                "size=256",
                "[0] = 999",
                "[1] = 998",
                "[2] = 997",
            ],
        )

        self.expect(
            "frame variable *ptr",
            matching=True,
            substrs=[
                "size=256",
                "[0] = 999",
                "[1] = 998",
                "[2] = 997",
            ],
        )

    @add_test_categories(["libstdcxx"])
    def test_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @add_test_categories(["libstdcxx"])
    def test_ptr_and_ref_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_ptr_and_ref()

    @add_test_categories(["libc++"])
    def test_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()

    @add_test_categories(["libc++"])
    def test_ptr_and_ref_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_ptr_and_ref()

    @add_test_categories(["msvcstl"])
    def test_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test()

    @add_test_categories(["msvcstl"])
    def test_ptr_and_ref_msvcstl(self):
        self.build()
        self.do_test_ptr_and_ref()



"""
Test that argdumper is a viable launching strategy.
"""
import os


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LaunchWithShellExpandTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(
        oslist=["windows", "linux", "freebsd"],
        bugnumber="llvm.org/pr24778 llvm.org/pr22627 llvm.org/pr48349",
    )
    @skipIfDarwinEmbedded  # iOS etc don't launch the binary via a shell, so arg expansion won't happen
    @expectedFailureNetBSD
    def test(self):
        self.build()
        target = self.createTestTarget()

        # Create any breakpoints we need
        breakpoint = target.BreakpointCreateBySourceRegex(
            "break here", lldb.SBFileSpec("main.cpp", False)
        )
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Ensure we do the expansion with /bin/sh on POSIX.
        os.environ["SHELL"] = "/bin/sh"

        self.runCmd(
            "process launch -X true -w %s -- fi*.tx? () > <" % (self.getSourceDir())
        )

        process = self.process()

        self.assertState(
            process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        thread = process.GetThreadAtIndex(0)

        self.assertTrue(
            thread.IsValid(), "Process stopped at 'main' should have a valid thread"
        )

        stop_reason = thread.GetStopReason()

        self.assertEqual(
            stop_reason,
            lldb.eStopReasonBreakpoint,
            "Thread in process stopped in 'main' should have a stop reason of eStopReasonBreakpoint",
        )

        self.expect_var_path("argv[1]", summary='"file1.txt"')
        self.expect_var_path("argv[2]", summary='"file2.txt"')
        self.expect_var_path("argv[3]", summary='"file3.txt"')
        self.expect_var_path("argv[4]", summary='"file4.txy"')
        self.expect_var_path("argv[5]", summary='"()"')
        self.expect_var_path("argv[6]", summary='">"')
        self.expect_var_path("argv[7]", summary='"<"')
        self.expect_var_path("argc", value="8")

        self.runCmd("process kill")

        self.runCmd('process launch -X true -w %s -- "foo bar"' % (self.getSourceDir()))

        process = self.process()

        self.assertState(
            process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        thread = process.GetThreadAtIndex(0)

        self.assertTrue(
            thread.IsValid(), "Process stopped at 'main' should have a valid thread"
        )

        stop_reason = thread.GetStopReason()

        self.assertEqual(
            stop_reason,
            lldb.eStopReasonBreakpoint,
            "Thread in process stopped in 'main' should have a stop reason of eStopReasonBreakpoint",
        )

        self.expect("frame variable argv[1]", substrs=["foo bar"])

        self.runCmd("process kill")

        self.runCmd(r"process launch -X true -w %s -- foo\ bar" % (self.getBuildDir()))

        process = self.process()

        self.assertState(
            process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        thread = process.GetThreadAtIndex(0)

        self.assertTrue(
            thread.IsValid(), "Process stopped at 'main' should have a valid thread"
        )

        stop_reason = thread.GetStopReason()

        self.assertEqual(
            stop_reason,
            lldb.eStopReasonBreakpoint,
            "Thread in process stopped in 'main' should have a stop reason of eStopReasonBreakpoint",
        )

        self.expect("frame variable argv[1]", substrs=["foo bar"])



"""
Test Getting the expression path for registers works correctly
"""

import lldb
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbtest import TestBase, VALID_BREAKPOINT, VALID_TARGET


class TestExprPathRegisters(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def verify_register_path(self, reg_value: lldb.SBValue):
        stream = lldb.SBStream()
        reg_name = reg_value.name
        self.assertTrue(
            reg_value.GetExpressionPath(stream),
            f"Expected an expression path for register {reg_name}.",
        )
        reg_expr_path = stream.GetData()
        self.assertEqual(reg_expr_path, f"${reg_name}")

    def test_float_registers(self):
        """Verify the expression path of the registers is valid."""
        self.build()
        _, _, thread, _ = lldbutil.run_to_name_breakpoint(self, "my_foo")
        frame = thread.GetSelectedFrame()
        self.assertTrue(frame, "Expected a valid Frame.")

        # possible floating point register on some cpus.
        register_names = [
            "xmm0",
            "ymm0",
            "v0",
            "v1",
            "f0",
            "f1",
            "d0",
            "d1",
            "vr0",
            "vr1",
            "st0",
            "st1",
        ]
        for name in register_names:
            reg_value = frame.FindRegister(name)
            # some the register will not be available for the cpu
            # only verify if it is valid.
            if reg_value:
                self.verify_register_path(reg_value)

    def test_all_registers(self):
        """Test all the registers that is avaiable on the machine"""
        self.build()
        _, _, thread, _ = lldbutil.run_to_name_breakpoint(self, "my_foo")
        frame = thread.GetSelectedFrame()
        self.assertTrue(frame, "Expected a valid Frame.")

        register_sets = frame.GetRegisters()
        self.assertTrue(register_sets.IsValid(), "Expected Frame Registers")

        for register_set in register_sets:
            for register in register_set.children:
                self.verify_register_path(register)



"""
Test the lldb disassemble command on lib stdc++.
"""

import os
import re
import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *


class StdCXXDisassembleTestCase(TestBase):
    @skipIfWindows
    def test_stdcxx_disasm(self):
        """Do 'disassemble' on each and every 'Code' symbol entry from the std c++ lib."""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line", lldb.SBFileSpec("main.cpp")
        )

        # Disassemble the functions on the call stack.
        self.runCmd("thread backtrace")
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)
        depth = thread.GetNumFrames()
        for i in range(depth - 1):
            frame = thread.GetFrameAtIndex(i)
            function = frame.GetFunction()
            if function.GetName():
                self.runCmd("disassemble -n '%s'" % function.GetName())

        lib_stdcxx = "FAILHORRIBLYHERE"
        # Find the stdc++ library...
        stdlib_regex = re.compile(r"/lib(std)?c\+\+")
        for module in target.module[stdlib_regex]:
            lib_stdcxx = module.file.fullpath
            break

        # At this point, lib_stdcxx is the full path to the stdc++ library and
        # module is the corresponding SBModule.

        if "lib" not in lib_stdcxx:
            self.skipTest(
                "This test requires libstdc++.so or libc++.dylib in the target's module list."
            )

        self.runCmd("image dump symtab '%s'" % lib_stdcxx)
        raw_output = self.res.GetOutput()
        # Now, look for every 'Code' symbol and feed its load address into the
        # command: 'disassemble -s load_address -e end_address', where the
        # end_address is taken from the next consecutive 'Code' symbol entry's
        # load address.
        #
        # The load address column comes after the file address column, with both
        # looks like '0xhhhhhhhh', i.e., 8 hexadecimal digits.
        codeRE = re.compile(
            r"""
                             \ Code\ {9}      # ' Code' followed by 9 SPCs,
                             0x[0-9a-f]{16}   # the file address column, and
                             \                # a SPC, and
                             (0x[0-9a-f]{16}) # the load address column, and
                             .*               # the rest.
                             """,
            re.VERBOSE,
        )
        # Maintain a start address variable; if we arrive at a consecutive Code
        # entry, then the load address of the that entry is fed as the end
        # address to the 'disassemble -s SA -e LA' command.
        SA = None
        for line in raw_output.split(os.linesep):
            match = codeRE.search(line)
            if match:
                LA = match.group(1)
                if self.TraceOn():
                    print("line:", line)
                    print("load address:", LA)
                    print("SA:", SA)
                if SA and LA:
                    if int(LA, 16) > int(SA, 16):
                        self.runCmd("disassemble -s %s -e %s" % (SA, LA))
                SA = LA
            else:
                # This entry is not a Code entry.  Reset SA = None.
                SA = None



# REQUIRES: host-supports-nvptx
# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.gpu as gpu
import mlir.dialects.gpu.passes
from mlir.passmanager import *


def run(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        f()
    return f


# CHECK-LABEL: testGPUToLLVMBin
@run
def testGPUToLLVMBin():
    with Context():
        module = Module.parse(
            r"""
module attributes {gpu.container_module} {
  gpu.module @kernel_module1 [#nvvm.target<chip = "sm_70">] {
    llvm.func @kernel(%arg0: i32, %arg1: !llvm.ptr,
        %arg2: !llvm.ptr, %arg3: i64, %arg4: i64,
        %arg5: i64) attributes {gpu.kernel} {
      llvm.return
    }
  }
}
    """
        )
    pm = PassManager("any")
    pm.add("gpu-module-to-binary{format=llvm}")
    pm.run(module.operation)
    # CHECK-LABEL: gpu.binary @kernel_module1
    print(module)

    o = gpu.ObjectAttr(module.body.operations[0].objects[0])
    # CHECK: #gpu.object<#nvvm.target<chip = "sm_70">, offload = "{{.*}}">
    print(o)
    # CHECK: #nvvm.target<chip = "sm_70">
    print(o.target)
    # CHECK: offload
    print(gpu.CompilationTarget(o.format))
    # CHECK: b'{{.*}}'
    print(o.object)
    # CHECK: None
    print(o.properties)


# CHECK-LABEL: testGPUToASMBin
@run
def testGPUToASMBin():
    with Context():
        module = Module.parse(
            r"""
module attributes {gpu.container_module} {
  gpu.module @kernel_module2 [#nvvm.target<flags = {fast}>, #nvvm.target] {
    llvm.func @kernel(%arg0: i32, %arg1: !llvm.ptr,
        %arg2: !llvm.ptr, %arg3: i64, %arg4: i64,
        %arg5: i64) attributes {gpu.kernel} {
      llvm.return
    }
  }
}
    """
        )
    pm = PassManager("any")
    pm.add("gpu-module-to-binary{format=isa}")
    pm.run(module.operation)
    # CHECK-LABEL:gpu.binary @kernel_module2
    print(module)

    o = gpu.ObjectAttr(module.body.operations[0].objects[0])
    # CHECK: #gpu.object<#nvvm.target<flags = {fast}>
    print(o)
    # CHECK: #nvvm.target<flags = {fast}>
    print(o.target)
    # CHECK: assembly
    print(gpu.CompilationTarget(o.format))
    # CHECK: b'//\n// Generated by LLVM NVPTX Back-End{{.*}}'
    print(o.object)
    # CHECK: {LLVMIRToISATimeInMs = {{[0-9]+}} : i64, O = 2 : i32}
    print(o.properties)



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetStopReason()
    obj.GetStopReasonDataCount()
    obj.GetStopReasonDataAtIndex(100)
    obj.GetStopDescription(256)
    obj.GetStopDescription(lldb.SBStream())
    obj.GetThreadID()
    obj.GetIndexID()
    obj.GetName()
    obj.GetQueueName()
    obj.StepOver(lldb.eOnlyDuringStepping)
    obj.StepInto(lldb.eOnlyDuringStepping)
    obj.StepOut()
    frame = lldb.SBFrame()
    obj.StepOutOfFrame(frame)
    obj.StepInstruction(True)
    filespec = lldb.SBFileSpec()
    obj.StepOverUntil(frame, filespec, 1234)
    obj.RunToAddress(0xABCD)
    obj.Suspend()
    obj.Resume()
    obj.IsSuspended()
    obj.GetNumFrames()
    obj.GetFrameAtIndex(200)
    obj.GetSelectedFrame()
    obj.SetSelectedFrame(999)
    obj.GetProcess()
    obj.GetDescription(lldb.SBStream())
    obj.Clear()
    for frame in obj:
        s = str(frame)
    len(obj)



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StdVBoolDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    TEST_WITH_PDB_DEBUG_INFO = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def do_test(self):
        """Test that that file and class static variables display correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=-1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)
            self.runCmd("settings set target.max-children-count 24", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.expect(
            "frame variable -A vBool",
            substrs=[
                "size=73",
                "[0] = false",
                "[1] = true",
                "[18] = false",
                "[27] = true",
                "[36] = false",
                "[47] = true",
                "[48] = true",
                "[49] = true",
                "[50] = false",
                "[56] = false",
                "[65] = true",
                "[70] = false",
                "[71] = true",
                "[72] = true",
            ],
        )

        self.expect(
            "expr -A -- vBool",
            substrs=[
                "size=73",
                "[0] = false",
                "[1] = true",
                "[18] = false",
                "[27] = true",
                "[36] = false",
                "[47] = true",
                "[48] = true",
                "[49] = true",
                "[50] = false",
                "[56] = false",
                "[65] = true",
                "[70] = false",
                "[71] = true",
                "[72] = true",
            ],
        )

    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()

    @add_test_categories(["libstdcxx"])
    def test_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @add_test_categories(["libstdcxx"])
    def test_libstdcxx_debug(self):
        self.build(
            dictionary={"USE_LIBSTDCPP": 1, "CXXFLAGS_EXTRAS": "-D_GLIBCXX_DEBUG"}
        )
        self.do_test()

    @add_test_categories(["msvcstl"])
    def test_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test()



"""
Test that breakpoints in an IT instruction don't fire if their condition is
false.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBreakpointIt(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(archs=no_match(["arm$"]))
    @skipIf(archs=["arm64", "arm64e", "arm64_32"])
    def test_false(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        self.runCmd("target create %s" % exe)
        lldbutil.run_break_set_by_symbol(
            self, "bkpt_false", extra_options="--skip-prologue 0"
        )

        self.runCmd("run")
        self.assertState(
            self.process().GetState(), lldb.eStateExited, "Breakpoint does not get hit"
        )

    @skipIf(archs=no_match(["arm$"]))
    @skipIf(archs=["arm64", "arm64e", "arm64_32"])
    def test_true(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        self.runCmd("target create %s" % exe)
        bpid = lldbutil.run_break_set_by_symbol(
            self, "bkpt_true", extra_options="--skip-prologue 0"
        )

        self.runCmd("run")
        self.assertIsNotNone(
            lldbutil.get_one_thread_stopped_at_breakpoint_id(self.process(), bpid)
        )



from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentTwoWatchpointsOneDelayBreakpoint(ConcurrentEventsBase):
    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple="^mips")
    @add_test_categories(["watchpoint"])
    def test(self):
        """Test two threads that trigger a watchpoint and one (1 second delay) breakpoint thread."""
        self.build()
        self.do_thread_actions(num_watchpoint_threads=2, num_delay_breakpoint_threads=1)



""" Test that evaluating expressions from within C++ lambdas works
    Particularly, we test the case of capturing "this" and
    using members of the captured object in expression evaluation
    while we're on a breakpoint inside a lambda.
"""


import lldb
from lldbsuite.test.lldbtest import *


class ExprInsideLambdaTestCase(TestBase):
    def expectExprError(self, expr: str, expected: str):
        frame = self.thread.GetFrameAtIndex(0)
        value = frame.EvaluateExpression(expr)
        errmsg = value.GetError().GetCString()
        self.assertIn(expected, errmsg)

    def test_expr_inside_lambda(self):
        """Test that lldb evaluating expressions inside lambda expressions works correctly."""
        self.build()
        (target, process, self.thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )

        # Inside 'Foo::method'

        # Check access to captured 'this'
        self.expect_expr("class_var", result_type="int", result_value="109")
        self.expect_expr("this->class_var", result_type="int", result_value="109")

        # Check that captured shadowed variables take preference over the
        # corresponding member variable
        self.expect_expr("shadowed", result_type="int", result_value="5")
        self.expect_expr("this->shadowed", result_type="int", result_value="-137")

        # Check access to local captures
        self.expect_expr("local_var", result_type="int", result_value="137")
        self.expect_expr("*class_ptr", result_type="int", result_value="137")

        # Check access to base class variables
        self.expect_expr("base_var", result_type="int", result_value="14")
        self.expect_expr("base_base_var", result_type="int", result_value="11")

        # Check access to global variable
        self.expect_expr("global_var", result_type="int", result_value="-5")

        # Check access to multiple captures/member variables
        self.expect_expr(
            "(shadowed + this->shadowed) * (base_base_var + local_var - class_var)",
            result_type="int",
            result_value="-5148",
        )

        # Check base-class function call
        self.expect_expr("baz_virt()", result_type="int", result_value="2")
        self.expect_expr("base_var", result_type="int", result_value="14")
        self.expect_expr("this->shadowed", result_type="int", result_value="-1")

        # 'p this' should yield 'struct Foo*'
        frame = self.thread.GetFrameAtIndex(0)
        outer_class_addr = frame.GetValueForVariablePath("this->this")
        self.expect_expr("this", result_value=outer_class_addr.GetValue())

        lldbutil.continue_to_breakpoint(process, bkpt)

        # Inside 'nested_lambda'

        # Check access to captured 'this'. Should still be 'struct Foo*'
        self.expect_expr("class_var", result_type="int", result_value="109")
        self.expect_expr("global_var", result_type="int", result_value="-5")
        self.expect_expr("this", result_value=outer_class_addr.GetValue())

        # Check access to captures
        self.expect_expr("lambda_local_var", result_type="int", result_value="5")
        self.expect_expr("local_var", result_type="int", result_value="137")

        # Check access to variable in previous frame which we didn't capture
        self.expectExprError("local_var_copy", "use of undeclared identifier")

        lldbutil.continue_to_breakpoint(process, bkpt)

        # By-ref mutates source variable
        self.expect_expr("lambda_local_var", result_type="int", result_value="0")

        # By-value doesn't mutate source variable
        self.expect_expr("local_var_copy", result_type="int", result_value="136")
        self.expect_expr("local_var", result_type="int", result_value="137")

        lldbutil.continue_to_breakpoint(process, bkpt)

        # Inside 'LocalLambdaClass::inner_method'

        # Check access to captured 'this'
        self.expect_expr("lambda_class_local", result_type="int", result_value="-12345")
        self.expect_expr(
            "this->lambda_class_local", result_type="int", result_value="-12345"
        )
        self.expect_expr("outer_ptr->class_var", result_type="int", result_value="109")

        # 'p this' should yield 'struct LocalLambdaClass*'
        frame = self.thread.GetFrameAtIndex(0)
        local_class_addr = frame.GetValueForVariablePath("this->this")
        self.assertNotEqual(local_class_addr, outer_class_addr)
        self.expect_expr("this", result_value=local_class_addr.GetValue())

        # Can still access global variable
        self.expect_expr("global_var", result_type="int", result_value="-5")

        # Check access to outer top-level structure's members
        self.expectExprError(
            "class_var",
            ("use of non-static data member" " 'class_var' of 'Foo' from nested type"),
        )

        self.expectExprError(
            "base_var", ("use of non-static data member" " 'base_var'")
        )

        self.expectExprError(
            "local_var",
            (
                "use of non-static data member 'local_var'"
                " of '(unnamed class)' from nested type 'LocalLambdaClass'"
            ),
        )

        # Inside non_capturing_method
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect_expr("local", result_type="const int", result_value="5")
        self.expect_expr("local2", result_type="const int", result_value="10")
        self.expect_expr("local2 * local", result_type="int", result_value="50")

        self.expectExprError(
            "class_var",
            ("use of non-static data member" " 'class_var' of 'Foo' from nested type"),
        )



"""
Tests that we correctly track AST layout info
(specifically alignment) when moving AST nodes
between ClangASTImporter instances (in this case,
from pch to executable to expression AST).
"""

import lldb
import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPchAlignment(TestBase):
    SHARED_BUILD_TESTCASE = False

    @add_test_categories(["gmodules"])
    def test_expr(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "return data", lldb.SBFileSpec("main.cpp")
        )

        self.expect(
            "frame variable data",
            substrs=["row = 1", "col = 2", "row = 3", "col = 4", "stride = 5"],
        )

    @add_test_categories(["gmodules"])
    def test_frame_var(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "return data", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr(
            "data",
            result_type="MatrixData",
            result_children=[
                ValueCheck(
                    name="section",
                    children=[
                        ValueCheck(
                            name="origin",
                            children=[
                                ValueCheck(name="row", value="1"),
                                ValueCheck(name="col", value="2"),
                            ],
                        ),
                        ValueCheck(
                            name="size",
                            children=[
                                ValueCheck(name="row", value="3"),
                                ValueCheck(name="col", value="4"),
                            ],
                        ),
                    ],
                ),
                ValueCheck(name="stride", value="5"),
            ],
        )



from collections import defaultdict
import lldb
import json
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *
import os


class TestTraceExport(TraceIntelPTTestCaseBase):
    def testErrorMessages(self):
        ctf_test_file = self.getBuildArtifact("ctf-test.json")
        # We first check the output when there are no targets
        self.expect(
            f"thread trace export ctf --file {ctf_test_file}",
            substrs=[
                "error: invalid target, create a target using the 'target create' command"
            ],
            error=True,
        )

        # We now check the output when there's a non-running target
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )

        self.expect(
            f"thread trace export ctf --file {ctf_test_file}",
            substrs=["error: Command requires a current process."],
            error=True,
        )

        # Now we check the output when there's a running target without a trace
        self.expect("b main")
        self.expect("run")

        self.expect(
            f"thread trace export ctf --file {ctf_test_file}",
            substrs=["error: Process is not being traced"],
            error=True,
        )

    def _testHtrBasicSuperBlockPassFullCheck(self):
        """
        Test the BasicSuperBlock pass of HTR.

        This test uses a very small trace so that the expected output is digestible and
        it's possible to manually verify the behavior of the algorithm.

        This test exhaustively checks that each entry
        in the output JSON is equal to the expected value.

        """

        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        ctf_test_file = self.getBuildArtifact("ctf-test.json")

        self.expect(f"thread trace export ctf --file {ctf_test_file}")
        self.assertTrue(os.path.exists(ctf_test_file))

        with open(ctf_test_file) as f:
            data = json.load(f)

        """
        The expected JSON contained by "ctf-test.json"

        dur: number of instructions in the block

        name: load address of the first instruction of the block and the
        name of the most frequently called function from the block (if applicable)

        ph: 'X' for Complete events (see link to documentation below)

        pid: the ID of the HTR layer the blocks belong to

        ts: offset from the beginning of the trace for the first instruction in the block

        See https://docs.google.com/document/d/1CvAClvFfyA5R-PhYUmn5OOQtYMH4h6I0nSsKchNAySU/preview#heading=h.j75x71ritcoy
        for documentation on the Trace Event Format
        """
        # Comments on the right indicate if a block is a "head" and/or "tail"
        # See BasicSuperBlockMerge in TraceHTR.h for a description of the algorithm
        expected = [
            {"dur": 1, "name": "0x400511", "ph": "X", "pid": 0, "ts": 0},
            {"dur": 1, "name": "0x400518", "ph": "X", "pid": 0, "ts": 1},
            {"dur": 1, "name": "0x40051f", "ph": "X", "pid": 0, "ts": 2},
            {"dur": 1, "name": "0x400529", "ph": "X", "pid": 0, "ts": 3},  # head
            {"dur": 1, "name": "0x40052d", "ph": "X", "pid": 0, "ts": 4},  # tail
            {"dur": 1, "name": "0x400521", "ph": "X", "pid": 0, "ts": 5},
            {"dur": 1, "name": "0x400525", "ph": "X", "pid": 0, "ts": 6},
            {"dur": 1, "name": "0x400529", "ph": "X", "pid": 0, "ts": 7},  # head
            {"dur": 1, "name": "0x40052d", "ph": "X", "pid": 0, "ts": 8},  # tail
            {"dur": 1, "name": "0x400521", "ph": "X", "pid": 0, "ts": 9},
            {"dur": 1, "name": "0x400525", "ph": "X", "pid": 0, "ts": 10},
            {"dur": 1, "name": "0x400529", "ph": "X", "pid": 0, "ts": 11},  # head
            {"dur": 1, "name": "0x40052d", "ph": "X", "pid": 0, "ts": 12},  # tail
            {"dur": 1, "name": "0x400521", "ph": "X", "pid": 0, "ts": 13},
            {"dur": 1, "name": "0x400525", "ph": "X", "pid": 0, "ts": 14},
            {"dur": 1, "name": "0x400529", "ph": "X", "pid": 0, "ts": 15},  # head
            {"dur": 1, "name": "0x40052d", "ph": "X", "pid": 0, "ts": 16},  # tail
            {"dur": 1, "name": "0x400521", "ph": "X", "pid": 0, "ts": 17},
            {"dur": 1, "name": "0x400525", "ph": "X", "pid": 0, "ts": 18},
            {"dur": 1, "name": "0x400529", "ph": "X", "pid": 0, "ts": 19},  # head
            {"dur": 1, "name": "0x40052d", "ph": "X", "pid": 0, "ts": 20},  # tail
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 3}},
                "dur": 3,
                "name": "0x400511",
                "ph": "X",
                "pid": 1,
                "ts": 0,
            },
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400529",
                "ph": "X",
                "pid": 1,
                "ts": 3,
            },  # head, tail
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400521",
                "ph": "X",
                "pid": 1,
                "ts": 5,
            },
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400529",
                "ph": "X",
                "pid": 1,
                "ts": 7,
            },  # head, tail
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400521",
                "ph": "X",
                "pid": 1,
                "ts": 9,
            },
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400529",
                "ph": "X",
                "pid": 1,
                "ts": 11,
            },  # head, tail
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400521",
                "ph": "X",
                "pid": 1,
                "ts": 13,
            },
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400529",
                "ph": "X",
                "pid": 1,
                "ts": 15,
            },  # head, tail
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400521",
                "ph": "X",
                "pid": 1,
                "ts": 17,
            },
            {
                "args": {"Metadata": {"Functions": [], "Number of Instructions": 2}},
                "dur": 2,
                "name": "0x400529",
                "ph": "X",
                "pid": 1,
                "ts": 19,
            },  # head, tail
        ]

        # Check that the length of the expected JSON array is equal to the actual
        self.assertEqual(len(data), len(expected))
        for i in range(len(data)):
            # Check each individual JSON object in "ctf-test.json" against the expected value above
            self.assertEqual(data[i], expected[i])

    def _testHtrBasicSuperBlockPassSequenceCheck(self):
        """
        Test the BasicSuperBlock pass of HTR.

        This test exports a modest sized trace and only checks that a particular sequence of blocks are
        expected, see `testHtrBasicSuperBlockPassFullCheck` for a more "exhaustive" test.

        TODO: Once the "trace save" command is implemented, gather Intel PT
        trace of this program and load it like the other tests instead of
        manually executing the commands to trace the program.
        """
        self.expect(
            f"target create {os.path.join(self.getSourceDir(), 'intelpt-trace', 'export_ctf_test_program.out')}"
        )
        self.expect("b main")
        self.expect("r")
        self.expect("b exit")
        self.expect("thread trace start")
        self.expect("c")

        ctf_test_file = self.getBuildArtifact("ctf-test.json")

        self.expect(f"thread trace export ctf --file {ctf_test_file}")
        self.assertTrue(os.path.exists(ctf_test_file))

        with open(ctf_test_file) as f:
            data = json.load(f)

        num_units_by_layer = defaultdict(int)
        index_of_first_layer_1_block = None
        for i, event in enumerate(data):
            layer_id = event.get("pid")
            self.assertIsNotNone(layer_id)
            if layer_id == 1 and index_of_first_layer_1_block is None:
                index_of_first_layer_1_block = i
            num_units_by_layer[layer_id] += 1

        # Check that there are only two layers and that the layer IDs are correct
        # Check that layer IDs are correct
        self.assertTrue(
            len(num_units_by_layer) == 2
            and 0 in num_units_by_layer
            and 1 in num_units_by_layer
        )

        # The expected block names for the first 7 blocks of layer 1
        expected_block_names = [
            "0x4005f0",
            "0x4005fe",
            "0x400606: iterative_handle_request_by_id(int, int)",
            "0x4005a7",
            "0x4005af",
            "0x4005b9: fast_handle_request(int)",
            "0x4005d5: log_response(int)",
        ]

        data_index = index_of_first_layer_1_block
        for i in range(len(expected_block_names)):
            self.assertEqual(data[data_index + i]["name"], expected_block_names[i])



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GenericMultiMapDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    TEST_WITH_PDB_DEBUG_INFO = True

    def setUp(self):
        TestBase.setUp(self)
        self.namespace = "std"

    def findVariable(self, name):
        var = self.frame().FindVariable(name)
        self.assertTrue(var.IsValid())
        return var

    def getVariableType(self, name):
        var = self.findVariable(name)
        return var.GetType().GetDisplayTypeName()

    def check(self, var_name, size):
        var = self.findVariable(var_name)
        self.assertEqual(var.GetNumChildren(), size)
        children = []
        for i in range(size):
            child = var.GetChildAtIndex(i)
            children.append(ValueCheck(value=child.GetValue()))
        self.expect_var_path(
            var_name, type=self.getVariableType(var_name), children=children
        )

    def do_test_with_run_command(self):
        """Test that that file and class static variables display correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        bkpt = self.target().FindBreakpointByID(
            lldbutil.run_break_set_by_source_regexp(
                self, "Set break point at this line."
            )
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        multimap = self.namespace + "::multimap"

        # We expect that in some malformed cases the map shows size 0
        self.expect("frame variable corrupt_map", substrs=[multimap, "size=0", "{}"])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable ii", substrs=[multimap, "size=0", "{}"])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable ii",
            substrs=[
                multimap,
                "size=2",
                "[0] = (first = 0, second = 0)",
                "[1] = (first = 1, second = 1)",
            ],
        )

        self.check("ii", 2)

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable ii",
            substrs=[
                multimap,
                "size=4",
                "[2] = ",
                "first = 2",
                "second = 0",
                "[3] = ",
                "first = 3",
                "second = 1",
            ],
        )

        self.check("ii", 4)

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable ii",
            substrs=[
                multimap,
                "size=8",
                "[5] = ",
                "first = 5",
                "second = 0",
                "[7] = ",
                "first = 7",
                "second = 1",
            ],
        )

        self.check("ii", 8)

        self.expect(
            "expression ii",
            substrs=[
                multimap,
                "size=8",
                "[5] = ",
                "first = 5",
                "second = 0",
                "[7] = ",
                "first = 7",
                "second = 1",
            ],
        )

        # check access-by-index
        self.expect("frame variable ii[0]", substrs=["first = 0", "second = 0"])
        self.expect("frame variable ii[3]", substrs=["first =", "second ="])

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("ii").MightHaveChildren(),
            "ii.MightHaveChildren() says False for non empty!",
        )

        # check that the expression parser does not make use of
        # synthetic children instead of running code
        # TOT clang has a fix for this, which makes the expression command here succeed
        # since this would make the test fail or succeed depending on clang version in use
        # this is safer commented for the time being
        # self.expect("expression ii[8]", matching=False, error=True,
        #            substrs = ['1234567'])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable ii", substrs=[multimap, "size=0", "{}"])

        self.expect("frame variable si", substrs=[multimap, "size=0", "{}"])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable si",
            substrs=[multimap, "size=1", "[0] = ", 'first = "zero"', "second = 0"],
        )

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable si",
            substrs=[
                multimap,
                "size=4",
                '[0] = (first = "one", second = 1)',
                '[1] = (first = "three", second = 3)',
                '[2] = (first = "two", second = 2)',
                '[3] = (first = "zero", second = 0)',
            ],
        )

        self.expect(
            "expression si",
            substrs=[
                multimap,
                "size=4",
                '[0] = (first = "one", second = 1)',
                '[1] = (first = "three", second = 3)',
                '[2] = (first = "two", second = 2)',
                '[3] = (first = "zero", second = 0)',
            ],
        )

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("si").MightHaveChildren(),
            "si.MightHaveChildren() says False for non empty!",
        )

        # check access-by-index
        self.expect("frame variable si[0]", substrs=["first = ", "one", "second = 1"])

        # check that the expression parser does not make use of
        # synthetic children instead of running code
        # TOT clang has a fix for this, which makes the expression command here succeed
        # since this would make the test fail or succeed depending on clang version in use
        # this is safer commented for the time being
        # self.expect("expression si[0]", matching=False, error=True,
        #            substrs = ['first = ', 'zero'])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable si", substrs=[multimap, "size=0", "{}"])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable is", substrs=[multimap, "size=0", "{}"])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable is",
            substrs=[
                multimap,
                "size=4",
                '[0] = (first = 1, second = "is")',
                '[1] = (first = 2, second = "smart")',
                '[2] = (first = 3, second = "!!!")',
                '[3] = (first = 85, second = "goofy")',
            ],
        )

        self.expect(
            "expression is",
            substrs=[
                multimap,
                "size=4",
                '[0] = (first = 1, second = "is")',
                '[1] = (first = 2, second = "smart")',
                '[2] = (first = 3, second = "!!!")',
                '[3] = (first = 85, second = "goofy")',
            ],
        )

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("is").MightHaveChildren(),
            "is.MightHaveChildren() says False for non empty!",
        )

        # check access-by-index
        self.expect("frame variable is[0]", substrs=["first = ", "second ="])

        # check that the expression parser does not make use of
        # synthetic children instead of running code
        # TOT clang has a fix for this, which makes the expression command here succeed
        # since this would make the test fail or succeed depending on clang version in use
        # this is safer commented for the time being
        # self.expect("expression is[0]", matching=False, error=True,
        #            substrs = ['first = ', 'goofy'])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable is", substrs=[multimap, "size=0", "{}"])

        self.check("is", 0)

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable ss", substrs=[multimap, "size=0", "{}"])

        self.check("ss", 0)

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect(
            "frame variable ss",
            substrs=[
                multimap,
                "size=3",
                '[0] = (first = "casa", second = "house")',
                '[1] = (first = "ciao", second = "hello")',
                '[2] = (first = "gatto", second = "cat")',
            ],
        )

        self.check("ss", 3)

        self.expect(
            "expression ss",
            substrs=[
                multimap,
                "size=3",
                '[0] = (first = "casa", second = "house")',
                '[1] = (first = "ciao", second = "hello")',
                '[2] = (first = "gatto", second = "cat")',
            ],
        )

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("ss").MightHaveChildren(),
            "ss.MightHaveChildren() says False for non empty!",
        )

        # check access-by-index
        self.expect("frame variable ss[2]", substrs=["gatto", "cat"])

        # check that the expression parser does not make use of
        # synthetic children instead of running code
        # TOT clang has a fix for this, which makes the expression command here succeed
        # since this would make the test fail or succeed depending on clang version in use
        # this is safer commented for the time being
        # self.expect("expression ss[3]", matching=False, error=True,
        #            substrs = ['gatto'])

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable ss", substrs=[multimap, "size=0", "{}"])

        self.check("ss", 0)

    @add_test_categories(["libstdcxx"])
    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_with_run_command_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_with_run_command()

    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    @add_test_categories(["libc++"])
    def test_with_run_command_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["msvcstl"])
    def test_with_run_command_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test_with_run_command()



"""
Test number of threads.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExitDuringBreakpointTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number for our breakpoint.
        self.breakpoint = line_number("main.cpp", "// Set breakpoint here")

    def test(self):
        """Test thread exit during breakpoint handling."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.breakpoint, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        # The exit probably occurred during breakpoint handling, but it isn't
        # guaranteed.  The main thing we're testing here is that the debugger
        # handles this cleanly is some way.

        # Get the number of threads
        num_threads = process.GetNumThreads()

        # Make sure we see at least five threads
        self.assertGreaterEqual(
            num_threads,
            5,
            "Number of expected threads and actual threads do not match.",
        )

        # Run to completion
        self.runCmd("continue")

        # At this point, the inferior process should have exited.
        self.assertEqual(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



"""Test the lldb public C++ api when doing multiple debug sessions simultaneously."""

import os
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestMultipleSimultaneousDebuggers(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    # Times out on heavily loaded Linux buildbots, don't want to get into tweaking
    # the timeout per bot. Does work when run alone. See:
    # https://github.com/llvm/llvm-project/issues/101162
    @skipIfLinux
    @skipIfNoSBHeaders
    @skipIfWindows
    @skipIfHostIncompatibleWithTarget
    def test_multiple_debuggers(self):
        self.driver_exe = self.getBuildArtifact("multi-process-driver")
        self.buildDriver(
            "multi-process-driver.cpp",
            self.driver_exe,
            defines=[("LLDB_HOST_ARCH", lldbplatformutil.getArchitecture())],
        )
        self.addTearDownHook(lambda: os.remove(self.driver_exe))

        self.inferior_exe = self.getBuildArtifact("testprog")
        self.buildDriver("testprog.cpp", self.inferior_exe)
        self.addTearDownHook(lambda: os.remove(self.inferior_exe))

        # check_call will raise a CalledProcessError if the executable doesn't
        # return exit code 0 to indicate success.  We can let this exception go
        # - the test harness will recognize it as a test failure.
        subprocess.check_call([self.driver_exe, self.inferior_exe])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        """
        This tests a static member with a type which size depends on the
        surrounding class. LLDB should *not* try to generate the record layout
        for those types while parsing the members from debug info.
        """
        self.build()
        self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        # Force the record layout for 'ToLayout' to be generated by printing
        # a value of it's type.
        self.expect("target variable test_var")



import os
import socket
import shutil
import lldbgdbserverutils
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPlatformProcessLaunchGDBServer(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def _launch_and_connect(self, exe):
        hostname = socket.getaddrinfo("localhost", 0, proto=socket.IPPROTO_TCP)[0][4][0]
        listen_url = "[%s]:0" % hostname

        port_file = self.getBuildArtifact("port")
        commandline_args = [
            "platform",
            "--listen",
            listen_url,
            "--socket-file",
            port_file,
        ]

        self.spawnSubprocess(exe, commandline_args)
        socket_id = lldbutil.wait_for_file_on_target(self, port_file)

        new_platform = lldb.SBPlatform("remote-" + self.getPlatform())
        self.dbg.SetSelectedPlatform(new_platform)

        connect_url = "connect://[%s]:%s" % (hostname, socket_id)
        self.runCmd("platform connect %s" % connect_url)

        wd = self.getBuildArtifact("wd")
        os.mkdir(wd)
        new_platform.SetWorkingDirectory(wd)

    @skipIfRemote
    # Windows cannot delete the executable while it is running.
    # On Darwin we may be using debugserver.
    @skipUnlessPlatform(["linux"])
    @add_test_categories(["lldb-server"])
    def test_launch_error(self):
        """
        Check that errors while handling qLaunchGDBServer are reported to the
        user.  Though this isn't a platform command in itself, the best way to
        test it is from Python because we can juggle multiple processes more
        easily.
        """

        self.build()

        # Run lldb-server from a new location.
        new_lldb_server = self.getBuildArtifact("lldb-server")
        shutil.copy(lldbgdbserverutils.get_lldb_server_exe(), new_lldb_server)
        self._launch_and_connect(new_lldb_server)

        # Now, remove our new lldb-server so that when it tries to invoke itself as a
        # gdbserver, it fails.
        os.remove(new_lldb_server)

        self.runCmd("target create {}".format(self.getBuildArtifact("a.out")))
        self.expect("run", substrs=["unable to launch a GDB server on"], error=True)

    @skipIfRemote
    @skipIfDarwin  # Uses debugserver for debugging
    @skipIfWindows
    @add_test_categories(["lldb-server"])
    def test_launch_with_unusual_process_name(self):
        """
        Test that lldb-server can launch a debug session when running under an
        unusual name (or under a symlink which resolves to an unusal name).
        """

        self.build()

        # Run lldb-server from a new location.
        new_lldb_server = self.getBuildArtifact("obfuscated-server")
        shutil.copy(lldbgdbserverutils.get_lldb_server_exe(), new_lldb_server)
        self._launch_and_connect(new_lldb_server)

        self.runCmd("target create {}".format(self.getBuildArtifact("a.out")))
        self.expect("run", substrs=["exited with status = 0"])



"""
Test calling user defined functions using expression evaluation.

Note:
  LLDBs current first choice of evaluating functions is using the IR interpreter,
  which is only supported on Hexagon. Otherwise JIT is used for the evaluation.

"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCommandCallUserDefinedFunction(TestBase):
    def test(self):
        """Test return values of user defined function calls."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Test recursive function call.
        self.expect_expr("fib(5)", result_type="unsigned int", result_value="5")

        # Test function with more than one parameter
        self.expect_expr("add(4, 8)", result_type="int", result_value="12")

        # Test nesting function calls in function parameters
        self.expect_expr("add(add(5,2),add(3,4))", result_type="int", result_value="14")
        self.expect_expr("add(add(5,2),fib(5))", result_type="int", result_value="12")

        # Test function with pointer parameter
        self.expect_expr(
            'stringCompare((const char*) "Hello world")',
            result_type="bool",
            result_value="true",
        )
        self.expect_expr(
            'stringCompare((const char*) "Hellworld")',
            result_type="bool",
            result_value="false",
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipIfRemote
    @skipIfWindows
    # glibc's dlopen doesn't support opening executables.
    # https://sourceware.org/bugzilla/show_bug.cgi?id=11754
    @skipIfLinux
    # freebsd's dlopen ditto
    @expectedFailureAll(oslist=["freebsd"])
    @expectedFailureNetBSD
    @no_debug_info_test
    def test(self):
        self.build()
        # Launch and stop before the dlopen call.
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        # Delete the breakpoint we no longer need.
        self.target().DeleteAllBreakpoints()

        # Check that the executable is the test binary.
        self.assertEqual(self.target().GetExecutable().GetFilename(), "a.out")

        # Continue so that dlopen is called.
        breakpoint = self.target().BreakpointCreateBySourceRegex(
            "// break after dlopen", lldb.SBFileSpec("main.c")
        )
        self.assertNotEqual(breakpoint.GetNumResolvedLocations(), 0)
        stopped_threads = lldbutil.continue_to_breakpoint(self.process(), breakpoint)
        self.assertEqual(len(stopped_threads), 1)

        # Check that the executable is still the test binary and not "other".
        self.assertEqual(self.target().GetExecutable().GetFilename(), "a.out")

        # Kill the process and run the program again.
        err = self.process().Kill()
        self.assertSuccess(err)

        # Test that we hit the breakpoint after dlopen.
        lldbutil.run_to_breakpoint_do_run(self, self.target(), breakpoint)



"""
Test my first lldb watchpoint.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class HelloWatchpointTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.c"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")
        # And the watchpoint variable declaration line number.
        self.decl = line_number(self.source, "// Watchpoint variable declaration.")
        self.exe_name = self.getBuildArtifact("a.out")
        self.d = {"C_SOURCES": self.source, "EXE": self.exe_name}

    @add_test_categories(["basic_process"])
    def test_hello_watchpoint_using_watchpoint_set(self):
        """Test a simple sequence of watchpoint creation and watchpoint hit."""
        self.build(dictionary=self.d)
        self.setTearDownCleanup(dictionary=self.d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped again due to the breakpoint.
        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Now let's set a write-type watchpoint for 'global'.
        # There should be only one watchpoint hit (see main.c).
        self.expect(
            "watchpoint set variable -w write global",
            WATCHPOINT_CREATED,
            substrs=[
                "Watchpoint created",
                "size = 4",
                "type = w",
                "%s:%d" % (self.source, self.decl),
            ],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v", substrs=["hit_count = 0"])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type), but
        # only once.  The stop reason of the thread should be watchpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "stop reason = watchpoint"],
        )

        self.runCmd("process continue")

        # Don't expect the read of 'global' to trigger a stop exception.
        process = self.dbg.GetSelectedTarget().GetProcess()
        if process.GetState() == lldb.eStateStopped:
            self.assertFalse(
                lldbutil.get_stopped_thread(process, lldb.eStopReasonWatchpoint)
            )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be 1.
        self.expect("watchpoint list -v", substrs=["hit_count = 1"])



""" Tests SBType.IsTypeComplete on Objective-C types. """

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.m")
        )

        # A class that is only forward declared is not complete.
        incomplete = self.expect_expr("incomplete", result_type="IncompleteClass *")
        self.assertTrue(incomplete.IsValid())
        incomplete_class = incomplete.GetType().GetPointeeType()
        self.assertTrue(incomplete_class.IsValid())
        self.assertFalse(incomplete_class.IsTypeComplete())

        # A class that has its interface fully declared is complete.
        complete = self.expect_expr("complete", result_type="CompleteClass *")
        self.assertTrue(complete.IsValid())
        complete_class = complete.GetType().GetPointeeType()
        self.assertTrue(complete_class.IsValid())
        self.assertTrue(complete_class.IsTypeComplete())

        # A class that has its interface fully declared and an implementation
        # is also complete.
        complete_with_impl = self.expect_expr(
            "complete_impl", result_type="CompleteClassWithImpl *"
        )
        self.assertTrue(complete_with_impl.IsValid())
        complete_class_with_impl = complete_with_impl.GetType().GetPointeeType()
        self.assertTrue(complete_class_with_impl.IsValid())
        self.assertTrue(complete_class_with_impl.IsTypeComplete())
